@@ -75,7 +75,7 @@
  *             v' = Av
  *          where v and v' are 1x3 column vectors in the form
  *             v = [x, y, 1]^    ^ denotes transpose
- *          and the affine tranlation matrix is
+ *          and the affine translation matrix is
  *             A = [ 1   0   tx
  *                   0   1   ty
  *                   0   0    1  ]
@@ -96,7 +96,6 @@ l_float32  *mat;
 
     if ((mat = (l_float32 *)LEPT_CALLOC(9, sizeof(l_float32))) == NULL)
         return (l_float32 *)ERROR_PTR("mat not made", procName, NULL);
-
     mat[0] = mat[4] = mat[8] = 1;
     mat[2] = transx;
     mat[5] = transy;
@@ -137,7 +136,6 @@ l_float32  *mat;
 
     if ((mat = (l_float32 *)LEPT_CALLOC(9, sizeof(l_float32))) == NULL)
         return (l_float32 *)ERROR_PTR("mat not made", procName, NULL);
-
     mat[0] = scalex;
     mat[4] = scaley;
     mat[8] = 1;
@@ -192,7 +190,6 @@ l_float32  *mat;
 
     if ((mat = (l_float32 *)LEPT_CALLOC(9, sizeof(l_float32))) == NULL)
         return (l_float32 *)ERROR_PTR("mat not made", procName, NULL);
-
     sina = sin(angle);
     cosa = cos(angle);
     mat[0] = mat[4] = cosa;
@@ -527,7 +524,7 @@ BOXA  *boxad;
  * \param[in]    size matrix is %size x %size; vectors are length %size
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 l_productMatVec(l_float32  *mat,
                 l_float32  *vecs,
                 l_float32  *vecd,
@@ -563,7 +560,7 @@ l_int32  i, j;
  * \param[in]    size of matrices
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 l_productMat2(l_float32  *mat1,
               l_float32  *mat2,
               l_float32  *matd,
@@ -602,7 +599,7 @@ l_int32  i, j, k, index;
  * \param[in]    size  of matrices
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 l_productMat3(l_float32  *mat1,
               l_float32  *mat2,
               l_float32  *mat3,
@@ -622,7 +619,8 @@ l_float32  *matt;
     if (!matd)
         return ERROR_INT("result matrix not defined", procName, 1);
 
-    if ((matt = (l_float32 *)LEPT_CALLOC(size * size, sizeof(l_float32))) == NULL)
+    if ((matt = (l_float32 *)LEPT_CALLOC((size_t)size * size,
+                                         sizeof(l_float32))) == NULL)
         return ERROR_INT("matt not made", procName, 1);
     l_productMat2(mat1, mat2, matt, size);
     l_productMat2(matt, mat3, matd, size);
@@ -642,7 +640,7 @@ l_float32  *matt;
  * \param[in]    size  of matrices
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 l_productMat4(l_float32  *mat1,
               l_float32  *mat2,
               l_float32  *mat3,
@@ -663,7 +661,8 @@ l_float32  *matt;
     if (!matd)
         return ERROR_INT("result matrix not defined", procName, 1);
 
-    if ((matt = (l_float32 *)LEPT_CALLOC(size * size, sizeof(l_float32))) == NULL)
+    if ((matt = (l_float32 *)LEPT_CALLOC((size_t)size * size,
+                                         sizeof(l_float32))) == NULL)
         return ERROR_INT("matt not made", procName, 1);
     l_productMat3(mat1, mat2, mat3, matt, size);
     l_productMat2(matt, mat4, matd, size);
