@@ -49,6 +49,10 @@
  *           l_int32   boxContainsPt()
  *           BOX      *boxaGetNearestToPt()
  *           BOX      *boxaGetNearestToLine()
+ *           l_int32   boxaFindNearestBoxes()
+ *           l_int32   boxaGetNearestByDirection()
+ *    static l_int32   boxHasOverlapInXorY()
+ *    static l_int32   boxGetDistanceInXorY()
  *           l_int32   boxIntersectByLine()
  *           l_int32   boxGetCenter()
  *           BOX      *boxClipToRectangle()
@@ -57,6 +61,7 @@
  *           BOXA     *boxaAdjustSides()
  *           BOX      *boxAdjustSides()
  *           BOXA     *boxaSetSide()
+ *           l_int32   boxSetSide()
  *           BOXA     *boxaAdjustWidthToTarget()
  *           BOXA     *boxaAdjustHeightToTarget()
  *           l_int32   boxEqual()
@@ -74,6 +79,11 @@
 
 #include "allheaders.h"
 
+static l_int32 boxHasOverlapInXorY(l_int32 c1, l_int32 s1, l_int32 c2,
+                                   l_int32 s2);
+static l_int32 boxGetDistanceInXorY(l_int32 c1, l_int32 s1, l_int32 c2,
+                                    l_int32 s2);
+
 
 /*---------------------------------------------------------------------*
  *                             Box geometry                            *
@@ -82,11 +92,11 @@
  * \brief   boxContains()
  *
  * \param[in]    box1, box2
- * \param[out]   presult 1 if box2 is entirely contained within
- *                       box1, and 0 otherwise
+ * \param[out]   presult     1 if box2 is entirely contained within box1;
+ *                           0 otherwise
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 boxContains(BOX     *box1,
             BOX     *box2,
             l_int32 *presult)
@@ -113,11 +123,11 @@ l_int32  x1, y1, w1, h1, x2, y2, w2, h2;
  * \brief   boxIntersects()
  *
  * \param[in]    box1, box2
- * \param[out]   presult 1 if any part of box2 is contained
- *                      in box1, and 0 otherwise
+ * \param[out]   presult    1 if any part of box2 is contained in box1;
+ *                          0 otherwise
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 boxIntersects(BOX      *box1,
               BOX      *box2,
               l_int32  *presult)
@@ -150,9 +160,9 @@ l_int32  l1, l2, r1, r2, t1, t2, b1, b2, w1, h1, w2, h2;
  * \brief   boxaContainedInBox()
  *
  * \param[in]    boxas
- * \param[in]    box for containment
- * \return  boxad boxa with all boxes in boxas that are
- *                     entirely contained in box, or NULL on error
+ * \param[in]    box     for containment
+ * \return  boxad  boxa with all boxes in boxas that are entirely
+ *                 contained in box, or NULL on error
  *
  * <pre>
  * Notes:
@@ -197,7 +207,7 @@ BOXA    *boxad;
  * \param[out]   pcount   number of boxes intersecting the box
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 boxaContainedInBoxCount(BOXA     *boxa,
                         BOX      *box,
                         l_int32  *pcount)
@@ -236,7 +246,7 @@ BOX     *box1;
  *                              some box in boxa1; 0 otherwise
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 boxaContainedInBoxa(BOXA     *boxa1,
                     BOXA     *boxa2,
                     l_int32  *pcontained)
@@ -279,9 +289,9 @@ BOX     *box1, *box2;
  * \brief   boxaIntersectsBox()
  *
  * \param[in]    boxas
- * \param[in]    box for intersecting
- * \return  boxad boxa with all boxes in boxas that intersect box,
- *                     or NULL on error
+ * \param[in]    box     for intersecting
+ * \return  boxad    boxa with all boxes in boxas that intersect box,
+ *                   or NULL on error
  *
  * <pre>
  * Notes:
@@ -327,7 +337,7 @@ BOXA    *boxad;
  * \param[out]   pcount   number of boxes intersecting the box
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 boxaIntersectsBoxCount(BOXA     *boxa,
                        BOX      *box,
                        l_int32  *pcount)
@@ -362,9 +372,8 @@ BOX     *box1;
  * \brief   boxaClipToBox()
  *
  * \param[in]    boxas
- * \param[in]    box for clipping
- * \return  boxad boxa with boxes in boxas clipped to box,
- *                     or NULL on error
+ * \param[in]    box     for clipping
+ * \return  boxad     boxa with boxes in boxas clipped to box, or NULL on error
  *
  * <pre>
  * Notes:
@@ -406,9 +415,8 @@ BOXA    *boxad;
  *
  * \param[in]       boxas
  * \param[in,out]   pixadb     debug output
- * \return  boxad where each set of boxes in boxas that overlap are
- *                     combined into a single bounding box in boxad, or
- *                     NULL on error.
+ * \return  boxad   where each set of boxes in boxas that overlap are combined
+ *                  into a single bounding box in boxad, or NULL on error.
  *
  * <pre>
  * Notes:
@@ -523,7 +531,7 @@ PIX     *pix1;
  *          to have only one operation on any rectangular region.
  * </pre>
  */
-l_int32
+l_ok
 boxaCombineOverlapsInPair(BOXA   *boxas1,
                           BOXA   *boxas2,
                           BOXA  **pboxad1,
@@ -533,7 +541,7 @@ boxaCombineOverlapsInPair(BOXA   *boxas1,
 l_int32  i, j, w, h, w2, h2, n1, n2, n1i, n2i, niters;
 l_int32  overlap, bigger, area1, area2;
 BOX     *box1, *box2, *box3;
-BOXA    *boxa1, *boxa2, *boxac1, *boxac2, *boxad1, *boxad2;
+BOXA    *boxa1, *boxa2, *boxac1, *boxac2;
 PIX     *pix1;
 
     PROCNAME("boxaCombineOverlapsInPair");
@@ -652,9 +660,9 @@ PIX     *pix1;
 /*!
  * \brief   boxOverlapRegion()
  *
- * \param[in]    box1, box2 two boxes
- * \return  box of overlap region between input boxes,
- *              or NULL if no overlap or on error
+ * \param[in]    box1, box2
+ * \return  box     of overlap region between input boxes;
+ *                  NULL if no overlap or on error
  *
  * <pre>
  * Notes:
@@ -694,9 +702,9 @@ l_int32  l1, l2, r1, r2, t1, t2, b1, b2, w1, h1, w2, h2, ld, td, rd, bd;
 /*!
  * \brief   boxBoundingRegion()
  *
- * \param[in]    box1, box2 two boxes
- * \return  box of bounding region containing the input boxes,
- *              or NULL on error
+ * \param[in]    box1, box2
+ * \return  box  of bounding region containing the input boxes;
+ *               NULL on error
  *
  * <pre>
  * Notes:
@@ -733,8 +741,8 @@ l_int32  l1, l2, r1, r2, t1, t2, b1, b2, w1, h1, w2, h2, ld, td, rd, bd;
 /*!
  * \brief   boxOverlapFraction()
  *
- * \param[in]    box1, box2 two boxes
- * \param[out]   pfract the fraction of box2 overlapped by box1
+ * \param[in]    box1, box2
+ * \param[out]   pfract      the fraction of box2 overlapped by box1
  * \return  0 if OK, 1 on error.
  *
  * <pre>
@@ -743,7 +751,7 @@ l_int32  l1, l2, r1, r2, t1, t2, b1, b2, w1, h1, w2, h2, ld, td, rd, bd;
  *          because the overlap is taken as a fraction of box2.
  * </pre>
  */
-l_int32
+l_ok
 boxOverlapFraction(BOX        *box1,
                    BOX        *box2,
                    l_float32  *pfract)
@@ -775,11 +783,11 @@ BOX     *boxo;
 /*!
  * \brief   boxOverlapArea()
  *
- * \param[in]    box1, box2 two boxes
- * \param[out]   parea the number of pixels in the overlap
+ * \param[in]    box1, box2
+ * \param[out]   parea       the number of pixels in the overlap
  * \return  0 if OK, 1 on error.
  */
-l_int32
+l_ok
 boxOverlapArea(BOX      *box1,
                BOX      *box2,
                l_int32  *parea)
@@ -811,13 +819,14 @@ BOX     *box;
  * \brief   boxaHandleOverlaps()
  *
  * \param[in]    boxas
- * \param[in]    op L_COMBINE, L_REMOVE_SMALL
- * \param[in]    range > 0, forward distance over which overlaps are checked
- * \param[in]    min_overlap minimum fraction of smaller box required for
- *                           overlap to count; 0.0 to ignore
- * \param[in]    max_ratio maximum fraction of small/large areas for
- *                         overlap to count; 1.0 to ignore
- * \param[out]   pnamap [optional] combining map
+ * \param[in]    op            L_COMBINE, L_REMOVE_SMALL
+ * \param[in]    range         forward distance over which overlaps
+ *                             are checked; > 0
+ * \param[in]    min_overlap   minimum fraction of smaller box required for
+ *                             overlap to count; 0.0 to ignore
+ * \param[in]    max_ratio     maximum fraction of small/large areas for
+ *                             overlap to count; 1.0 to ignore
+ * \param[out]   pnamap        [optional] combining map
  * \return  boxad, or NULL on error.
  *
  * <pre>
@@ -947,9 +956,9 @@ NUMA      *namap;
 /*!
  * \brief   boxSeparationDistance()
  *
- * \param[in]    box1, box2 two boxes, in any order
- * \param[out]   ph_sep [optional] horizontal separation
- * \param[out]   pv_sep [optional] vertical separation
+ * \param[in]    box1, box2    two boxes, in any order
+ * \param[out]   ph_sep        [optional] horizontal separation
+ * \param[out]   pv_sep        [optional] vertical separation
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -960,7 +969,7 @@ NUMA      *namap;
  *          a distance d, the returned separation is -d.
  * </pre>
  */
-l_int32
+l_ok
 boxSeparationDistance(BOX      *box1,
                       BOX      *box2,
                       l_int32  *ph_sep,
@@ -1008,7 +1017,7 @@ l_int32  l1, t1, w1, h1, r1, b1, l2, t2, w2, h2, r2, b2;
  * \param[in]    type     L_SORT_BY_WIDTH, L_SORT_BY_HEIGHT,
  *                        L_SORT_BY_MAX_DIMENSION, L_SORT_BY_PERIMETER,
  *                        L_SORT_BY_AREA,
- * \param[out]   prel   1 if box1 > box2, 0 if the same, -1 if box1 < box2
+ * \param[out]   prel     1 if box1 > box2, 0 if the same, -1 if box1 < box2
  * \return   0 if OK, 1 on error
  *
  * <pre>
@@ -1016,7 +1025,7 @@ l_int32  l1, t1, w1, h1, r1, b1, l2, t2, w2, h2, r2, b2;
  *      (1) We're re-using the SORT enum for these comparisons.
  * </pre>
  */
-l_int32
+l_ok
 boxCompareSize(BOX      *box1,
                BOX      *box2,
                l_int32   type,
@@ -1063,11 +1072,11 @@ l_int32  w1, h1, w2, h2, size1, size2;
  * \brief   boxContainsPt()
  *
  * \param[in]    box
- * \param[in]    x, y a point
- * \param[out]   pcontains 1 if box contains point; 0 otherwise
+ * \param[in]    x, y        a point
+ * \param[out]   pcontains   1 if box contains point; 0 otherwise
  * \return  0 if OK, 1 on error.
  */
-l_int32
+l_ok
 boxContainsPt(BOX       *box,
               l_float32  x,
               l_float32  y,
@@ -1093,9 +1102,9 @@ l_int32  bx, by, bw, bh;
  * \brief   boxaGetNearestToPt()
  *
  * \param[in]    boxa
- * \param[in]    x, y  point
- * \return  box with centroid closest to the given point [x,y],
- *              or NULL if no boxes in boxa
+ * \param[in]    x, y    point
+ * \return  box   with centroid closest to the given point [x,y],
+ *                or NULL if no boxes in boxa
  *
  * <pre>
  * Notes:
@@ -1142,8 +1151,8 @@ BOX       *box;
  *
  * \param[in]    boxa
  * \param[in]    x, y   (y = -1 for vertical line; x = -1 for horiz line)
- * \return  box with centroid closest to the given line,
- *              or NULL if no boxes in boxa
+ * \return  box  with centroid closest to the given line,
+ *               or NULL if no boxes in boxa
  *
  * <pre>
  * Notes:
@@ -1194,6 +1203,236 @@ BOX       *box;
 }
 
 
+/*!
+ * \brief   boxaFindNearestBoxes()
+ *
+ * \param[in]    boxa         either unsorted, or 2D sorted in LR/TB scan order
+ * \param[in]    dist_select  L_NON_NEGATIVE, L_ALL
+ * \param[in]    range        search distance from box i; use 0 to search
+ *                            entire boxa (e.g., if it's not 2D sorted)
+ * \param[out]   pnaaindex    for each box in %boxa, contains a numa of 4
+ *                            box indices (per direction) of the nearest box
+ * \param[out]   pnaadist     for each box in %boxa, this contains a numa
+ * \return  0 if OK, 1 on error
+ * <pre>
+ * Notes:
+ *      (1) See boxaGetNearestByDirection() for usage of %dist_select
+ *          and %range.
+ * </pre>
+ */
+l_ok
+boxaFindNearestBoxes(BOXA     *boxa,
+                     l_int32   dist_select,
+                     l_int32   range,
+                     NUMAA   **pnaaindex,
+                     NUMAA   **pnaadist)
+{
+l_int32  i, n, index, dist;
+NUMA    *nai, *nad;
+NUMAA   *naai, *naad;
+
+    PROCNAME("boxaFindNearestBoxes");
+
+    if (pnaaindex) *pnaaindex = NULL;
+    if (pnaadist) *pnaadist = NULL;
+    if (!pnaaindex)
+        return ERROR_INT("&naaindex not defined", procName, 1);
+    if (!pnaadist)
+        return ERROR_INT("&naadist not defined", procName, 1);
+    if (!boxa)
+        return ERROR_INT("boxa not defined", procName, 1);
+
+    n = boxaGetCount(boxa);
+    naai = numaaCreate(n);
+    naad = numaaCreate(n);
+    *pnaaindex = naai;
+    *pnaadist = naad;
+    for (i = 0; i < n; i++) {
+        nai = numaCreate(4);
+        nad = numaCreate(4);
+        boxaGetNearestByDirection(boxa, i, L_FROM_LEFT, dist_select,
+                                  range, &index, &dist);
+        numaAddNumber(nai, index);
+        numaAddNumber(nad, dist);
+        boxaGetNearestByDirection(boxa, i, L_FROM_RIGHT, dist_select,
+                                  range, &index, &dist);
+        numaAddNumber(nai, index);
+        numaAddNumber(nad, dist);
+        boxaGetNearestByDirection(boxa, i, L_FROM_TOP, dist_select,
+                                  range, &index, &dist);
+        numaAddNumber(nai, index);
+        numaAddNumber(nad, dist);
+        boxaGetNearestByDirection(boxa, i, L_FROM_BOT, dist_select,
+                                  range, &index, &dist);
+        numaAddNumber(nai, index);
+        numaAddNumber(nad, dist);
+        numaaAddNuma(naai, nai, L_INSERT);
+        numaaAddNuma(naad, nad, L_INSERT);
+    }
+    return 0;
+}
+
+
+/*!
+ * \brief   boxaGetNearestByDirection()
+ *
+ * \param[in]    boxa         either unsorted, or 2D sorted in LR/TB scan order
+ * \param[in]    i            box we test against
+ * \param[in]    dir          direction to look: L_FROM_LEFT, L_FROM_RIGHT,
+ *                            L_FROM_TOP, L_FROM_BOT
+ * \param[in]    dist_select  L_NON_NEGATIVE, L_ALL
+ * \param[in]    range        search distance from box i; use 0 to search
+ *                            entire boxa (e.g., if it's not 2D sorted)
+ * \param[out]   pindex       index in boxa of nearest box with overlapping
+ *                            coordinates in the indicated direction;
+ *                            -1 if there is no box
+ * \param[out]   pdist        distance of the nearest box in the indicated
+ *                            direction; 100000 if no box
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) For efficiency, use a LR/TD sorted %boxa, which can be
+ *          made by flattening a 2D sorted boxaa.  In that case,
+ *          %range can be some positive integer like 50.
+ *      (2) If boxes overlap, the distance will be < 0.  Use %dist_select
+ *          to determine if these should count or not.  If L_ALL, then
+ *          one box will match as the nearest to another in 2 or more
+ *          directions.
+ * </pre>
+ */
+l_ok
+boxaGetNearestByDirection(BOXA     *boxa,
+                          l_int32   i,
+                          l_int32   dir,
+                          l_int32   dist_select,
+                          l_int32   range,
+                          l_int32  *pindex,
+                          l_int32  *pdist)
+{
+l_int32  j, jmin, jmax, n, mindist, dist, index;
+l_int32  x, y, w, h, bx, by, bw, bh;
+
+    PROCNAME("boxaGetNearestByDirection");
+
+    if (pindex) *pindex = -1;
+    if (pdist) *pdist = 100000;
+    if (!pindex)
+        return ERROR_INT("&index not defined", procName, 1);
+    if (!pdist)
+        return ERROR_INT("&dist not defined", procName, 1);
+    if (!boxa)
+        return ERROR_INT("boxa not defined", procName, 1);
+    if (dir != L_FROM_LEFT && dir != L_FROM_RIGHT &&
+        dir != L_FROM_TOP && dir != L_FROM_BOT)
+        return ERROR_INT("invalid dir", procName, 1);
+    if (dist_select != L_NON_NEGATIVE && dist_select != L_ALL)
+        return ERROR_INT("invalid dist_select", procName, 1);
+    n = boxaGetCount(boxa);
+    if (i < 0 || i >= n)
+        return ERROR_INT("invalid box index", procName, 1);
+
+    jmin = (range <= 0) ? 0 : L_MAX(0, i - range);
+    jmax = (range <= 0) ? n - 1 : L_MIN(n -1, i + range);
+    boxaGetBoxGeometry(boxa, i, &x, &y, &w, &h);
+    mindist = 100000;
+    index = -1;
+    if (dir == L_FROM_LEFT || dir == L_FROM_RIGHT) {
+        for (j = jmin; j <= jmax; j++) {
+            if (j == i) continue;
+            boxaGetBoxGeometry(boxa, j, &bx, &by, &bw, &bh);
+            if ((bx >= x && dir == L_FROM_LEFT) ||  /* not to the left */
+                (x >= bx && dir == L_FROM_RIGHT))   /* not to the right */
+                continue;
+            if (boxHasOverlapInXorY(y, h, by, bh) == 1) {
+                dist = boxGetDistanceInXorY(x, w, bx, bw);
+                if (dist_select == L_NON_NEGATIVE && dist < 0) continue;
+                if (dist < mindist) {
+                    mindist = dist;
+                    index = j;
+                }
+            }
+        }
+    } else if (dir == L_FROM_TOP || dir == L_FROM_BOT) {
+        for (j = jmin; j <= jmax; j++) {
+            if (j == i) continue;
+            boxaGetBoxGeometry(boxa, j, &bx, &by, &bw, &bh);
+            if ((by >= y && dir == L_FROM_TOP) ||  /* not above */
+                (y >= by && dir == L_FROM_BOT))   /* not below */
+                continue;
+            if (boxHasOverlapInXorY(x, w, bx, bw) == 1) {
+                dist = boxGetDistanceInXorY(y, h, by, bh);
+                if (dist_select == L_NON_NEGATIVE && dist < 0) continue;
+                if (dist < mindist) {
+                    mindist = dist;
+                    index = j;
+                }
+            }
+        }
+    }
+    *pindex = index;
+    *pdist = mindist;
+    return 0;
+}
+
+
+/*!
+ * \brief   boxHasOverlapInXorY()
+ *
+ * \param[in]    c1   left or top coordinate of box1
+ * \param[in]    s1   width or height of box1
+ * \param[in]    c2   left or top coordinate of box2
+ * \param[in]    s2   width or height of box2
+ * \return  0 if no overlap; 1 if any overlap
+ *
+ * <pre>
+ * Notes:
+ *      (1) Like boxGetDistanceInXorY(), this is used for overlaps both in
+ *          x (which projected vertically) and in y (projected horizontally)
+ * </pre>
+ */
+static l_int32
+boxHasOverlapInXorY(l_int32  c1,
+                    l_int32  s1,
+                    l_int32  c2,
+                    l_int32  s2)
+{
+l_int32  ovlp;
+
+    if (c1 > c2)
+        ovlp = c2 + s2 - 1 - c1;
+    else
+        ovlp = c1 + s1 - 1 - c2;
+    return (ovlp < 0) ? 0 : 1;
+}
+
+
+/*!
+ * \brief   boxGetDistanceInXorY()
+ *
+ * \param[in]    c1   left or top coordinate of box1
+ * \param[in]    s1   width or height of box1
+ * \param[in]    c2   left or top coordinate of box2
+ * \param[in]    s2   width or height of box2
+ * \return  distance between them (if < 0, box2 overlaps box1 in the
+ *                                 dimension considered)
+ */
+static l_int32
+boxGetDistanceInXorY(l_int32  c1,
+                     l_int32  s1,
+                     l_int32  c2,
+                     l_int32  s2)
+{
+l_int32  dist;
+
+    if (c1 > c2)
+        dist = c1 - (c2 + s2 - 1);
+    else
+        dist = c2 - (c1 + s1 - 1);
+    return dist;
+}
+
+
 /*!
  * \brief   boxGetCenter()
  *
@@ -1201,7 +1440,7 @@ BOX       *box;
  * \param[out]   pcx, pcy location of center of box
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 boxGetCenter(BOX        *box,
              l_float32  *pcx,
              l_float32  *pcy)
@@ -1242,7 +1481,7 @@ l_int32  x, y, w, h;
  *      (2) Represent a vertical line by one with a large but finite slope.
  * </pre>
  */
-l_int32
+l_ok
 boxIntersectByLine(BOX       *box,
                    l_int32    x,
                    l_int32    y,
@@ -1335,7 +1574,7 @@ PTA       *pta;
  * \param[in]    box
  * \param[in]    wi, hi rectangle representing image
  * \return  part of box within given rectangle, or NULL on error
- *              or if box is entirely outside the rectangle
+ *          or if box is entirely outside the rectangle
  *
  * <pre>
  * Notes:
@@ -1401,7 +1640,7 @@ BOX  *boxd;
  *                     ....
  * </pre>
  */
-l_int32
+l_ok
 boxClipToRectangleParams(BOX      *box,
                          l_int32   w,
                          l_int32   h,
@@ -1451,7 +1690,7 @@ BOX     *boxc;
  * \param[in]    loc new location of the side that is changing
  * \param[in]    sideflag L_FROM_LEFT, etc., indicating the side that moves
  * \return  boxd, or NULL on error or if the computed boxd has
- *              width or height <= 0.
+ *          width or height <= 0.
  *
  * <pre>
  * Notes:
@@ -1543,13 +1782,13 @@ BOXA    *boxad;
 /*!
  * \brief   boxAdjustSides()
  *
- * \param[in]    boxd  [optional]; this can be null, equal to boxs,
- *                     or different from boxs
- * \param[in]    boxs  starting box; to have sides adjusted
- * \param[in]    delleft, delright, deltop, delbot changes in location of
- *                                                 each side
+ * \param[in]    boxd     [optional]; this can be null, equal to boxs,
+ *                        or different from boxs
+ * \param[in]    boxs     starting box; to have sides adjusted
+ * \param[in]    delleft, delright, deltop, delbot    changes in location
+ *                                                    of each side
  * \return  boxd, or NULL on error or if the computed boxd has
- *              width or height <= 0.
+ *          width or height <= 0.
  *
  * <pre>
  * Notes:
@@ -1600,11 +1839,11 @@ l_int32  x, y, w, h, xl, xr, yt, yb, wnew, hnew;
 /*!
  * \brief   boxaSetSide()
  *
- * \param[in]    boxad use NULL to get a new one; same as boxas for in-place
+ * \param[in]    boxad    use NULL to get a new one; same as boxas for in-place
  * \param[in]    boxas
- * \param[in]    side L_SET_LEFT, L_SET_RIGHT, L_SET_TOP, L_SET_BOT
- * \param[in]    val location to set for given side, for each box
- * \param[in]    thresh min abs difference to cause resetting to %val
+ * \param[in]    side     L_SET_LEFT, L_SET_RIGHT, L_SET_TOP, L_SET_BOT
+ * \param[in]    val      location to set for given side, for each box
+ * \param[in]    thresh   min abs difference to cause resetting to %val
  * \return  boxad, or NULL on error
  *
  * <pre>
@@ -1623,7 +1862,7 @@ boxaSetSide(BOXA    *boxad,
             l_int32  val,
             l_int32  thresh)
 {
-l_int32  x, y, w, h, n, i, diff;
+l_int32  n, i;
 BOX     *box;
 
     PROCNAME("boxaSetSide");
@@ -1643,39 +1882,78 @@ BOX     *box;
     n = boxaGetCount(boxad);
     for (i = 0; i < n; i++) {
         box = boxaGetBox(boxad, i, L_CLONE);
-        boxGetGeometry(box, &x, &y, &w, &h);
-        if (side == L_SET_LEFT) {
-            diff = x - val;
-            if (L_ABS(diff) >= thresh)
-                boxSetGeometry(box, val, y, w + diff, h);
-        } else if (side == L_SET_RIGHT) {
-            diff = x + w -1 - val;
-            if (L_ABS(diff) >= thresh)
-                boxSetGeometry(box, x, y, val - x + 1, h);
-        } else if (side == L_SET_TOP) {
-            diff = y - val;
-            if (L_ABS(diff) >= thresh)
-                boxSetGeometry(box, x, val, w, h + diff);
-        } else { /* side == L_SET_BOT */
-            diff = y + h - 1 - val;
-            if (L_ABS(diff) >= thresh)
-                boxSetGeometry(box, x, y, w, val - y + 1);
-        }
-        boxDestroy(&box);
+        boxSetSide(box, side, val, thresh);
+        boxDestroy(&box);  /* the clone */
     }
 
     return boxad;
 }
 
 
+/*!
+ * \brief   boxSetSide()
+ *
+ * \param[in]    boxs
+ * \param[in]    side     L_SET_LEFT, L_SET_RIGHT, L_SET_TOP, L_SET_BOT
+ * \param[in]    val      location to set for given side, for each box
+ * \param[in]    thresh   min abs difference to cause resetting to %val
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) In-place operation.
+ *      (2) Use %thresh = 0 to definitely set the side to %val.
+ * </pre>
+ */
+l_ok 
+boxSetSide(BOX     *boxs,
+           l_int32  side,
+           l_int32  val,
+           l_int32  thresh)
+{
+l_int32  x, y, w, h, diff;
+
+    PROCNAME("boxSetSide");
+
+    if (!boxs)
+        return ERROR_INT("box not defined", procName, 1);
+    if (side != L_SET_LEFT && side != L_SET_RIGHT &&
+        side != L_SET_TOP && side != L_SET_BOT)
+        return ERROR_INT("invalid side", procName, 1);
+    if (val < 0)
+        return ERROR_INT("val < 0", procName, 1);
+
+    boxGetGeometry(boxs, &x, &y, &w, &h);
+    if (side == L_SET_LEFT) {
+        diff = x - val;
+        if (L_ABS(diff) >= thresh)
+            boxSetGeometry(boxs, val, y, w + diff, h);
+    } else if (side == L_SET_RIGHT) {
+        diff = x + w -1 - val;
+        if (L_ABS(diff) >= thresh)
+            boxSetGeometry(boxs, x, y, val - x + 1, h);
+    } else if (side == L_SET_TOP) {
+        diff = y - val;
+        if (L_ABS(diff) >= thresh)
+            boxSetGeometry(boxs, x, val, w, h + diff);
+    } else { /* side == L_SET_BOT */
+        diff = y + h - 1 - val;
+        if (L_ABS(diff) >= thresh)
+            boxSetGeometry(boxs, x, y, w, val - y + 1);
+    }
+
+    return 0;
+}
+
+
 /*!
  * \brief   boxaAdjustWidthToTarget()
  *
- * \param[in]    boxad use NULL to get a new one; same as boxas for in-place
+ * \param[in]    boxad    use NULL to get a new one; same as boxas for in-place
  * \param[in]    boxas
- * \param[in]    sides L_ADJUST_LEFT, L_ADJUST_RIGHT, L_ADJUST_LEFT_AND_RIGHT
- * \param[in]    target target width if differs by more than thresh
- * \param[in]    thresh min abs difference in width to cause adjustment
+ * \param[in]    sides    L_ADJUST_LEFT, L_ADJUST_RIGHT, L_ADJUST_LEFT_AND_RIGHT
+ * \param[in]    target   target width if differs by more than thresh
+ * \param[in]    thresh   min abs difference in width to cause adjustment
  * \return  boxad, or NULL on error
  *
  * <pre>
@@ -1738,11 +2016,11 @@ BOX     *box;
 /*!
  * \brief   boxaAdjustHeightToTarget()
  *
- * \param[in]    boxad use NULL to get a new one
+ * \param[in]    boxad    use NULL to get a new one
  * \param[in]    boxas
- * \param[in]    sides L_ADJUST_TOP, L_ADJUST_BOT, L_ADJUST_TOP_AND_BOT
- * \param[in]    target target height if differs by more than thresh
- * \param[in]    thresh min abs difference in height to cause adjustment
+ * \param[in]    sides    L_ADJUST_TOP, L_ADJUST_BOT, L_ADJUST_TOP_AND_BOT
+ * \param[in]    target   target height if differs by more than thresh
+ * \param[in]    thresh   min abs difference in height to cause adjustment
  * \return  boxad, or NULL on error
  *
  * <pre>
@@ -1811,10 +2089,10 @@ BOX     *box;
  *
  * \param[in]    box1
  * \param[in]    box2
- * \param[out]   psame 1 if equal; 0 otherwise
+ * \param[out]   psame    1 if equal; 0 otherwise
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 boxEqual(BOX      *box1,
          BOX      *box2,
          l_int32  *psame)
@@ -1839,8 +2117,8 @@ boxEqual(BOX      *box1,
  * \param[in]    boxa1
  * \param[in]    boxa2
  * \param[in]    maxdist
- * \param[out]   pnaindex [optional] index array of correspondences
- * \param[out]   psame (1 if equal; 0 otherwise
+ * \param[out]   pnaindex     [optional] index array of correspondences
+ * \param[out]   psame        1 if equal; 0 otherwise
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1861,7 +2139,7 @@ boxEqual(BOX      *box1,
  *          boxa are equal.
  * </pre>
  */
-l_int32
+l_ok
 boxaEqual(BOXA     *boxa1,
           BOXA     *boxa2,
           l_int32   maxdist,
@@ -1885,7 +2163,8 @@ NUMA     *na;
     if (n != boxaGetCount(boxa2))
         return 0;
 
-    countarray = (l_int32 *)LEPT_CALLOC(n, sizeof(l_int32));
+    if ((countarray = (l_int32 *)LEPT_CALLOC(n, sizeof(l_int32))) == NULL)
+        return ERROR_INT("calloc fail for countarray", procName, 1);
     na = numaMakeConstant(0.0, n);
 
     for (i = 0; i < n; i++) {
@@ -1929,7 +2208,7 @@ NUMA     *na;
  * \param[in]    box1
  * \param[in]    box2
  * \param[in]    leftdiff, rightdiff, topdiff, botdiff
- * \param[out]   psimilar 1 if similar; 0 otherwise
+ * \param[out]   psimilar   1 if similar; 0 otherwise
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1939,7 +2218,7 @@ NUMA     *na;
  *          pairs differ by more than this amount, the boxes are not similar.
  * </pre>
  */
-l_int32
+l_ok
 boxSimilar(BOX      *box1,
            BOX      *box2,
            l_int32   leftdiff,
@@ -1980,9 +2259,9 @@ l_int32  l1, l2, r1, r2, t1, t2, b1, b2;
  * \param[in]    boxa1
  * \param[in]    boxa2
  * \param[in]    leftdiff, rightdiff, topdiff, botdiff
- * \param[in]    debug output details of non-similar boxes
- * \param[out]   psimilar 1 if similar; 0 otherwise
- * \param[out]   pnasim [optional] na containing 1 if similar; else 0
+ * \param[in]    debug      output details of non-similar boxes
+ * \param[out]   psimilar   1 if similar; 0 otherwise
+ * \param[out]   pnasim     [optional] na containing 1 if similar; else 0
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1994,7 +2273,7 @@ l_int32  l1, l2, r1, r2, t1, t2, b1, b2;
  *          after failure.
  * </pre>
  */
-l_int32
+l_ok
 boxaSimilar(BOXA     *boxa1,
             BOXA     *boxa2,
             l_int32   leftdiff,
@@ -2054,10 +2333,10 @@ BOX     *box1, *box2;
 /*!
  * \brief   boxaJoin()
  *
- * \param[in]    boxad  dest boxa; add to this one
- * \param[in]    boxas  source boxa; add from this one
- * \param[in]    istart  starting index in boxas
- * \param[in]    iend  ending index in boxas; use -1 to cat all
+ * \param[in]    boxad     dest boxa; add to this one
+ * \param[in]    boxas     source boxa; add from this one
+ * \param[in]    istart    starting index in boxas
+ * \param[in]    iend      ending index in boxas; use -1 to cat all
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -2068,7 +2347,7 @@ BOX     *box1, *box2;
  *      (4) if boxas == NULL or has no boxes, this is a no-op.
  * </pre>
  */
-l_int32
+l_ok
 boxaJoin(BOXA    *boxad,
          BOXA    *boxas,
          l_int32  istart,
@@ -2103,10 +2382,10 @@ BOX     *box;
 /*!
  * \brief   boxaaJoin()
  *
- * \param[in]    baad  dest boxaa; add to this one
- * \param[in]    baas  source boxaa; add from this one
- * \param[in]    istart  starting index in baas
- * \param[in]    iend  ending index in baas; use -1 to cat all
+ * \param[in]    baad     dest boxaa; add to this one
+ * \param[in]    baas     source boxaa; add from this one
+ * \param[in]    istart   starting index in baas
+ * \param[in]    iend     ending index in baas; use -1 to cat all
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -2117,7 +2396,7 @@ BOX     *box;
  *      (4) if baas == NULL, this is a no-op.
  * </pre>
  */
-l_int32
+l_ok
 boxaaJoin(BOXAA   *baad,
           BOXAA   *baas,
           l_int32  istart,
@@ -2154,9 +2433,9 @@ BOXA    *boxa;
  * \brief   boxaSplitEvenOdd()
  *
  * \param[in]    boxa
- * \param[in]    fillflag 1 to put invalid boxes in place; 0 to omit
- * \param[out]   pboxae, pboxao save even and odd boxes in their
- *                 separate boxa, setting the other type to invalid boxes.
+ * \param[in]    fillflag         1 to put invalid boxes in place; 0 to omit
+ * \param[out]   pboxae, pboxao   save even and odd boxes in their separate
+ *                                boxa, setting the other type to invalid boxes.
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -2167,7 +2446,7 @@ BOXA    *boxa;
  *      (2) If %fillflag == 0, boxae has only copies of the even boxes.
  * </pre>
  */
-l_int32
+l_ok
 boxaSplitEvenOdd(BOXA    *boxa,
                  l_int32  fillflag,
                  BOXA   **pboxae,
@@ -2217,9 +2496,9 @@ BOX     *box, *boxt;
 /*!
  * \brief   boxaMergeEvenOdd()
  *
- * \param[in]    boxae boxes to go in even positions in merged boxa
- * \param[in]    boxao boxes to go in odd positions in merged boxa
- * \param[in]    fillflag 1 if there are invalid boxes in placeholders
+ * \param[in]    boxae       boxes to go in even positions in merged boxa
+ * \param[in]    boxao       boxes to go in odd positions in merged boxa
+ * \param[in]    fillflag    1 if there are invalid boxes in placeholders
  * \return  boxad merged, or NULL on error
  *
  * <pre>
