@@ -34,29 +34,29 @@
  *      4- and 8-connected components: counts, bounding boxes and images
  *
  *      Top-level calls:
- *           BOXA     *pixConnComp()
- *           BOXA     *pixConnCompPixa()
- *           BOXA     *pixConnCompBB()
- *           l_int32   pixCountConnComp()
+ *            BOXA     *pixConnComp()
+ *            BOXA     *pixConnCompPixa()
+ *            BOXA     *pixConnCompBB()
+ *            l_int32   pixCountConnComp()
  *
  *      Identify the next c.c. to be erased:
- *           l_int32   nextOnPixelInRaster()
- *           l_int32   nextOnPixelInRasterLow()
+ *            l_int32   nextOnPixelInRaster()
+ *    static  l_int32   nextOnPixelInRasterLow()
  *
  *      Erase the c.c., saving the b.b.:
- *           BOX      *pixSeedfillBB()
- *           BOX      *pixSeedfill4BB()
- *           BOX      *pixSeedfill8BB()
+ *            BOX      *pixSeedfillBB()
+ *            BOX      *pixSeedfill4BB()
+ *            BOX      *pixSeedfill8BB()
  *
  *      Just erase the c.c.:
- *           l_int32   pixSeedfill()
- *           l_int32   pixSeedfill4()
- *           l_int32   pixSeedfill8()
+ *            l_int32   pixSeedfill()
+ *            l_int32   pixSeedfill4()
+ *            l_int32   pixSeedfill8()
  *
  *      Static stack helper functions for single raster line seedfill:
- *           static void    pushFillsegBB()
- *           static void    pushFillseg()
- *           static void    popFillseg()
+ *            static void    pushFillsegBB()
+ *            static void    pushFillseg()
+ *            static void    popFillseg()
  *
  *  The basic method in pixConnCompBB() is very simple.  We scan the
  *  image in raster order, looking for the next ON pixel.  When it
@@ -105,6 +105,9 @@ struct FillSeg
 };
 typedef struct FillSeg    FILLSEG;
 
+static l_int32 nextOnPixelInRasterLow(l_uint32 *data, l_int32 w, l_int32 h,
+                                      l_int32 wpl, l_int32 xstart,
+                                      l_int32 ystart, l_int32 *px, l_int32 *py);
 
     /* Static accessors for FillSegs on a stack */
 static void pushFillsegBB(L_STACK *stack, l_int32 xleft, l_int32 xright,
@@ -149,10 +152,8 @@ pixConnComp(PIX     *pixs,
     PROCNAME("pixConnComp");
 
     if (ppixa) *ppixa = NULL;
-    if (!pixs)
-        return (BOXA *)ERROR_PTR("pixs not defined", procName, NULL);
-    if (pixGetDepth(pixs) != 1)
-        return (BOXA *)ERROR_PTR("pixs not 1 bpp", procName, NULL);
+    if (!pixs || pixGetDepth(pixs) != 1)
+        return (BOXA *)ERROR_PTR("pixs undefined or not 1 bpp", procName, NULL);
     if (connectivity != 4 && connectivity != 8)
         return (BOXA *)ERROR_PTR("connectivity not 4 or 8", procName, NULL);
 
@@ -218,6 +219,7 @@ L_STACK  *stack, *auxstack;
     if (iszero)
         return boxaCreate(1);  /* return empty boxa and empty pixa */
 
+    pixSetPadBits(pixs, 0);
     pix1 = pixCopy(NULL, pixs);
     pix2 = pixCopy(NULL, pixs);
     if (!pix1 || !pix2) {
@@ -266,7 +268,8 @@ L_STACK  *stack, *auxstack;
 #if  DEBUG
     pixCountPixels(pix1, &iszero, NULL);
     fprintf(stderr, "Number of remaining pixels = %d\n", iszero);
-    pixWrite("junkremain", pix1, IFF_PNG);
+    lept_mkdir("lept/cc");
+    pixWriteDebug("/tmp/lept/cc/remain.png", pix1, IFF_PNG);
 #endif  /* DEBUG */
 
         /* Remove old boxa of pixa and replace with a copy */
@@ -324,6 +327,7 @@ L_STACK  *stack, *auxstack;
     if (iszero)
         return boxaCreate(1);  /* return empty boxa */
 
+    pixSetPadBits(pixs, 0);
     if ((pix1 = pixCopy(NULL, pixs)) == NULL)
         return (BOXA *)ERROR_PTR("pix1 not made", procName, NULL);
 
@@ -356,7 +360,8 @@ L_STACK  *stack, *auxstack;
 #if  DEBUG
     pixCountPixels(pix1, &iszero, NULL);
     fprintf(stderr, "Number of remaining pixels = %d\n", iszero);
-    pixWrite("junkremain", pix1, IFF_PNG);
+    lept_mkdir("lept/cc");
+    pixWriteDebug("/tmp/lept/cc/remain.png", pix1, IFF_PNG);
 #endif  /* DEBUG */
 
         /* Cleanup, freeing the fillsegs on each stack */
@@ -381,7 +386,7 @@ cleanup:
  *     2 It works on a copy of the input pix.  The c.c. are located
  *         in raster order and erased one at a time.
  */
-l_int32
+l_ok
 pixCountConnComp(PIX      *pixs,
                  l_int32   connectivity,
                  l_int32  *pcount)
@@ -406,6 +411,7 @@ L_STACK  *stack, *auxstack;
     if (iszero)
         return 0;
 
+    pixSetPadBits(pixs, 0);
     if ((pix1 = pixCopy(NULL, pixs)) == NULL)
         return ERROR_INT("pix1 not made", procName, 1);
     h = pixGetHeight(pixs);
@@ -477,7 +483,7 @@ l_uint32  *data;
  * \param[out]   px, py  coord value of next ON pixel
  * \return  1 if a pixel is found; 0 otherwise or on error
  */
-l_int32
+static l_int32
 nextOnPixelInRasterLow(l_uint32  *data,
                        l_int32    w,
                        l_int32    h,
@@ -838,7 +844,7 @@ BOX       *box;
  *      (2) See pixSeedfill4() and pixSeedfill8() for details.
  * </pre>
  */
-l_int32
+l_ok
 pixSeedfill(PIX      *pixs,
             L_STACK  *stack,
             l_int32   x,
@@ -882,7 +888,7 @@ l_int32  retval;
  *      (3) Reference: see pixSeedFill4BB()
  * </pre>
  */
-l_int32
+l_ok
 pixSeedfill4(PIX      *pixs,
              L_STACK  *stack,
              l_int32   x,
@@ -976,7 +982,7 @@ l_uint32  *data, *line;
  *      (3) Reference: see pixSeedFill8BB()
  * </pre>
  */
-l_int32
+l_ok
 pixSeedfill8(PIX      *pixs,
              L_STACK  *stack,
              l_int32   x,
