@@ -77,7 +77,7 @@ static l_int32 pixRenderHorizEndPoints(PIX *pixs, PTA *ptal, PTA *ptar,
 
 
 #ifndef  NO_CONSOLE_IO
-#define  DEBUG_TEXTLINE_CENTERS    0   /* set this to 1 for debuging */
+#define  DEBUG_TEXTLINE_CENTERS    0   /* set this to 1 for debugging */
 #define  DEBUG_SHORT_LINES         0   /* ditto */
 #else
 #define  DEBUG_TEXTLINE_CENTERS    0   /* always must be 0 */
@@ -144,7 +144,7 @@ static const l_float32   L_ALLOWED_W_FRACT = 0.05;  /* no bigger */
  *          longest textlines.
  * </pre>
  */
-l_int32
+l_ok
 dewarpBuildPageModel(L_DEWARP    *dew,
                      const char  *debugfile)
 {
@@ -165,7 +165,7 @@ PTAA    *ptaa1, *ptaa2;
         lept_rmdir("lept/dewmod");  /* erase previous images */
         lept_mkdir("lept/dewmod");
         pixDisplayWithTitle(pixs, 0, 0, "pixs", 1);
-        pixWrite("/tmp/lept/dewmod/0010.png", pixs, IFF_PNG);
+        pixWriteDebug("/tmp/lept/dewmod/0010.png", pixs, IFF_PNG);
     }
 
         /* Make initial estimate of centers of textlines */
@@ -179,7 +179,7 @@ PTAA    *ptaa1, *ptaa2;
         pta = generatePtaFilledCircle(1);
         pix2 = pixGenerateFromPta(pta, 5, 5);
         pix3 = pixDisplayPtaaPattern(NULL, pix1, ptaa1, pix2, 2, 2);
-        pixWrite("/tmp/lept/dewmod/0020.png", pix3, IFF_PNG);
+        pixWriteDebug("/tmp/lept/dewmod/0020.png", pix3, IFF_PNG);
         pixDestroy(&pix1);
         pixDestroy(&pix2);
         pixDestroy(&pix3);
@@ -195,7 +195,7 @@ PTAA    *ptaa1, *ptaa2;
         pta = generatePtaFilledCircle(1);
         pix2 = pixGenerateFromPta(pta, 5, 5);
         pix3 = pixDisplayPtaaPattern(NULL, pix1, ptaa2, pix2, 2, 2);
-        pixWrite("/tmp/lept/dewmod/0030.png", pix3, IFF_PNG);
+        pixWriteDebug("/tmp/lept/dewmod/0030.png", pix3, IFF_PNG);
         pixDestroy(&pix1);
         pixDestroy(&pix2);
         pixDestroy(&pix3);
@@ -243,12 +243,12 @@ PTAA    *ptaa1, *ptaa2;
     if (debugfile) {
         dewarpPopulateFullRes(dew, NULL, 0, 0);
         pix1 = fpixRenderContours(dew->fullvdispar, 3.0, 0.15);
-        pixWrite("/tmp/lept/dewmod/0060.png", pix1, IFF_PNG);
+        pixWriteDebug("/tmp/lept/dewmod/0060.png", pix1, IFF_PNG);
         pixDisplay(pix1, 1000, 0);
         pixDestroy(&pix1);
         if (ret == 0) {
             pix1 = fpixRenderContours(dew->fullhdispar, 3.0, 0.15);
-            pixWrite("/tmp/lept/dewmod/0070.png", pix1, IFF_PNG);
+            pixWriteDebug("/tmp/lept/dewmod/0070.png", pix1, IFF_PNG);
             pixDisplay(pix1, 1000, 0);
             pixDestroy(&pix1);
         }
@@ -290,13 +290,13 @@ PTAA    *ptaa1, *ptaa2;
  *          a pdf.  Non-pix debug output goes to /tmp.
  * </pre>
  */
-l_int32
+l_ok
 dewarpFindVertDisparity(L_DEWARP  *dew,
                         PTAA      *ptaa,
                         l_int32    rotflag)
 {
 l_int32     i, j, nlines, npts, nx, ny, sampling;
-l_float32   c0, c1, c2, x, y, midy, val, medval, medvar, minval, maxval;
+l_float32   c0, c1, c2, x, y, midy, val, medval, meddev, minval, maxval;
 l_float32  *famidys;
 NUMA       *nax, *nafit, *nacurve0, *nacurve1, *nacurves;
 NUMA       *namidy, *namidys, *namidysi;
@@ -361,7 +361,7 @@ FPIX       *fpix;
         pta = generatePtaFilledCircle(1);
         pixcirc = pixGenerateFromPta(pta, 5, 5);
         pix2 = pixDisplayPtaaPattern(NULL, pix1, ptaat, pixcirc, 2, 2);
-        pixWrite("/tmp/lept/dewmod/0041.png", pix2, IFF_PNG);
+        pixWriteDebug("/tmp/lept/dewmod/0041.png", pix2, IFF_PNG);
         pixDestroy(&pix1);
         pixDestroy(&pix2);
         ptaDestroy(&pta);
@@ -374,15 +374,15 @@ FPIX       *fpix;
          * the line curvatures.  It is not rejecting lines based on
          * the magnitude of the curvature.  That is done when constraints
          * are applied for valid models. */
-    numaGetMedianVariation(nacurve0, &medval, &medvar);
+    numaGetMedianDevFromMedian(nacurve0, &medval, &meddev);
     L_INFO("\nPage %d\n", procName, dew->pageno);
-    L_INFO("Pass 1: Curvature: medval = %f, medvar = %f\n",
-           procName, medval, medvar);
+    L_INFO("Pass 1: Curvature: medval = %f, meddev = %f\n",
+           procName, medval, meddev);
     ptaa1 = ptaaCreate(nlines);
     nacurve1 = numaCreate(nlines);
     for (i = 0; i < nlines; i++) {  /* for each line */
         numaGetFValue(nacurve0, i, &val);
-        if (L_ABS(val - medval) > 7.0 * medvar)  /* TODO: reduce to ~ 3.0 */
+        if (L_ABS(val - medval) > 7.0 * meddev)  /* TODO: reduce to ~ 3.0 */
             continue;
         pta = ptaaGetPta(ptaa0, i, L_CLONE);
         ptaaAddPta(ptaa1, pta, L_INSERT);
@@ -423,8 +423,8 @@ FPIX       *fpix;
     numaDestroy(&nacurve1);
     numaDestroy(&namidysi);
     if (dew->debug) {
-        numaWrite("/tmp/lept/dewdebug/midys.na", namidys);
-        numaWrite("/tmp/lept/dewdebug/curves.na", nacurves);
+        numaWriteDebug("/tmp/lept/dewdebug/midys.na", namidys);
+        numaWriteDebug("/tmp/lept/dewdebug/curves.na", nacurves);
         pix1 = pixConvertTo32(pixdb);
         ptacirc = generatePtaFilledCircle(5);
         pixcirc = pixGenerateFromPta(ptacirc, 11, 11);
@@ -433,7 +433,7 @@ FPIX       *fpix;
         srand(3);  /* use the same colors for text and reference lines */
         pixRenderMidYs(pix1, namidys, 2);
         pix2 = (rotflag) ? pixRotateOrth(pix1, 3) : pixClone(pix1);
-        pixWrite("/tmp/lept/dewmod/0042.png", pix2, IFF_PNG);
+        pixWriteDebug("/tmp/lept/dewmod/0042.png", pix2, IFF_PNG);
         pixDisplay(pix2, 0, 0);
         ptaDestroy(&ptacirc);
         pixDestroy(&pixcirc);
@@ -459,7 +459,7 @@ FPIX       *fpix;
         ptaDestroy(&pta);
     }
     if (dew->debug) {
-        ptaaWrite("/tmp/lept/dewdebug/ptaa3.ptaa", ptaa3, 0);
+        ptaaWriteDebug("/tmp/lept/dewdebug/ptaa3.ptaa", ptaa3, 0);
     }
 
         /* Generate ptaa4 by taking vertical 'columns' from ptaa3.
@@ -480,7 +480,7 @@ FPIX       *fpix;
         ptaaAddPta(ptaa4, pta, L_INSERT);
     }
     if (dew->debug) {
-        ptaaWrite("/tmp/lept/dewdebug/ptaa4.ptaa", ptaa4, 0);
+        ptaaWriteDebug("/tmp/lept/dewdebug/ptaa4.ptaa", ptaa4, 0);
     }
 
         /* Do quadratic fit vertically on each of the pixel columns
@@ -505,7 +505,7 @@ FPIX       *fpix;
         ptaDestroy(&pta);
     }
     if (dew->debug) {
-        ptaaWrite("/tmp/lept/dewdebug/ptaa5.ptaa", ptaa5, 0);
+        ptaaWriteDebug("/tmp/lept/dewdebug/ptaa5.ptaa", ptaa5, 0);
         convertFilesToPdf("/tmp/lept/dewmod", "004", 135, 1.0, 0, 0,
                           "Dewarp Vert Disparity",
                           "/tmp/lept/dewarp/vert_disparity.pdf");
@@ -554,11 +554,11 @@ FPIX       *fpix;
  *      (5) Debug output goes to /tmp/lept/dewmod/ for collection into a pdf.
  * </pre>
  */
-l_int32
+l_ok
 dewarpFindHorizDisparity(L_DEWARP  *dew,
                          PTAA      *ptaa)
 {
-l_int32    i, j, n, w, h, nx, ny, sampling, ret;
+l_int32    i, j, h, nx, ny, sampling, ret;
 l_float32  c0, c1, cl0, cl1, cl2, cr0, cr1, cr2;
 l_float32  x, y, refl, refr;
 l_float32  val, mederr;
@@ -591,8 +591,8 @@ FPIX      *fpix;
     if (dew->debug) {
         lept_mkdir("lept/dewdebug");
         lept_mkdir("lept/dewarp");
-        ptaWrite("/tmp/lept/dewdebug/endpts_left1.pta", ptal1, 1);
-        ptaWrite("/tmp/lept/dewdebug/endpts_right1.pta", ptar1, 1);
+        ptaWriteDebug("/tmp/lept/dewdebug/endpts_left1.pta", ptal1, 1);
+        ptaWriteDebug("/tmp/lept/dewdebug/endpts_right1.pta", ptar1, 1);
     }
 
         /* Filter the points by x-location to prevent 2-column images
@@ -663,7 +663,7 @@ FPIX      *fpix;
         pixDisplayPta(pix1, pix1, pta2);
         pixRenderHorizEndPoints(pix1, ptal2, ptar2, 0xff000000);
         pixDisplay(pix1, 600, 800);
-        pixWrite("/tmp/lept/dewmod/0051.png", pix1, IFF_PNG);
+        pixWriteDebug("/tmp/lept/dewmod/0051.png", pix1, IFF_PNG);
         pixDestroy(&pix1);
 
         pix1 = pixDisplayPta(NULL, dew->pixs, pta1);
@@ -672,7 +672,7 @@ FPIX      *fpix;
         ptarft = ptaTranspose(ptar3);
         pixRenderHorizEndPoints(pix1, ptalft, ptarft, 0x0000ff00);
         pixDisplay(pix1, 800, 800);
-        pixWrite("/tmp/lept/dewmod/0052.png", pix1, IFF_PNG);
+        pixWriteDebug("/tmp/lept/dewmod/0052.png", pix1, IFF_PNG);
         convertFilesToPdf("/tmp/lept/dewmod", "005", 135, 1.0, 0, 0,
                           "Dewarp Horiz Disparity",
                           "/tmp/lept/dewarp/horiz_disparity.pdf");
@@ -801,9 +801,9 @@ PTAA     *ptaa;
 
     if (debugflag) {
         lept_mkdir("lept/dewmod");
-        pixWrite("/tmp/lept/dewmod/0011.tif", pix1, IFF_TIFF_G4);
+        pixWriteDebug("/tmp/lept/dewmod/0011.tif", pix1, IFF_TIFF_G4);
         pixDisplayWithTitle(pix1, 0, 600, "pix1", 1);
-        pixWrite("/tmp/lept/dewmod/0012.tif", pix2, IFF_TIFF_G4);
+        pixWriteDebug("/tmp/lept/dewmod/0012.tif", pix2, IFF_TIFF_G4);
         pixDisplayWithTitle(pix2, 0, 800, "pix2", 1);
     }
     pixDestroy(&pix1);
@@ -827,7 +827,7 @@ PTAA     *ptaa;
     }
     if (debugflag) {
         pix2 = pixaDisplay(pixa2, w, h);
-        pixWrite("/tmp/lept/dewmod/0013.tif", pix2, IFF_TIFF_G4);
+        pixWriteDebug("/tmp/lept/dewmod/0013.tif", pix2, IFF_TIFF_G4);
         pixDisplayWithTitle(pix2, 0, 1000, "pix2", 1);
         pixDestroy(&pix2);
     }
@@ -846,7 +846,7 @@ PTAA     *ptaa;
     if (debugflag) {
         pix1 = pixCreateTemplate(pixs);
         pix2 = pixDisplayPtaa(pix1, ptaa);
-        pixWrite("/tmp/lept/dewmod/0014.tif", pix2, IFF_PNG);
+        pixWriteDebug("/tmp/lept/dewmod/0014.tif", pix2, IFF_PNG);
         pixDisplayWithTitle(pix2, 0, 1200, "pix3", 1);
         pixDestroy(&pix1);
         pixDestroy(&pix2);
@@ -1107,8 +1107,8 @@ PTA       *ptal1, *ptar1, *ptal2, *ptar2;
         }
     }
     if (dew->debug) {
-        ptaWrite("/tmp/lept/dewdebug/endpts_left2.pta", ptal1, 1);
-        ptaWrite("/tmp/lept/dewdebug/endpts_right2.pta", ptar1, 1);
+        ptaWriteDebug("/tmp/lept/dewdebug/endpts_left2.pta", ptal1, 1);
+        ptaWriteDebug("/tmp/lept/dewdebug/endpts_right2.pta", ptar1, 1);
     }
 
     n = L_MIN(ptaGetCount(ptal1), ptaGetCount(ptar1));
@@ -1132,8 +1132,8 @@ PTA       *ptal1, *ptar1, *ptal2, *ptar2;
         return 1;
     }
     if (dew->debug) {
-        ptaWrite("/tmp/lept/dewdebug/endpts_left3.pta", ptal2, 1);
-        ptaWrite("/tmp/lept/dewdebug/endpts_right3.pta", ptar2, 1);
+        ptaWriteDebug("/tmp/lept/dewdebug/endpts_left3.pta", ptal2, 1);
+        ptaWriteDebug("/tmp/lept/dewdebug/endpts_right3.pta", ptar2, 1);
     }
 
     *pptalf = ptal2;
@@ -1195,7 +1195,7 @@ PTA       *ptau1, *ptau2, *ptad1, *ptad2;
     }
 
         /* Check the lower half */
-    ptad1 = ptaSelectRange(ptas, n / 2 + 1, 0);
+    ptad1 = ptaSelectRange(ptas, n / 2 + 1, -1);
     ptaGetRankValue(ptad1, 0.5, NULL, L_SORT_BY_Y, &rval);
     nd = ptaGetCount(ptad1);
     ptad2 = ptaCreate(nd);
@@ -1352,7 +1352,7 @@ NUMA      *naerr;
  *          disparity only if the absolute value of the fractional
  *          difference equals or exceeds this threshold.
  *      (2) %parity indicates where the binding is: on the left for
- *          %parity == 0 and on the right for @parity == 1.
+ *          %parity == 0 and on the right for %parity == 1.
  *      (3) This takes a 1 bpp %pixb where both vertical and horizontal
  *          disparity have been applied, so the text lines are straight and,
  *          more importantly, the line end points are vertically aligned.
@@ -1374,7 +1374,7 @@ NUMA      *naerr;
  *      (6) Debug output goes to /tmp/lept/dewmod/ for collection into a pdf.
  * </pre>
  */
-l_int32
+l_ok
 dewarpFindHorizSlopeDisparity(L_DEWARP  *dew,
                               PIX       *pixb,
                               l_float32  fractthresh,
@@ -1426,7 +1426,7 @@ FPIX      *fpix;
         lept_mkdir("lept/dew");
         gplotSimple1(na1, GPLOT_PNG, "/tmp/lept/dew/0091", NULL);
         lept_mv("/tmp/lept/dew/0091.png", "lept/dewmod", NULL, NULL);
-        pixWrite("/tmp/lept/dewmod/0090.png", pix1, IFF_PNG);
+        pixWriteDebug("/tmp/lept/dewmod/0090.png", pix1, IFF_PNG);
     }
     pixDestroy(&pix1);
     boxaDestroy(&boxa2);
@@ -1602,7 +1602,7 @@ FPIX      *fpix;
  *          See notes there.
  * </pre>
  */
-l_int32
+l_ok
 dewarpBuildLineModel(L_DEWARP    *dew,
                      l_int32      opensize,
                      const char  *debugfile)
@@ -1634,7 +1634,7 @@ PTAA    *ptaa1, *ptaa2;
         lept_mkdir("lept/dewmod");
         lept_mkdir("lept/dewarp");
         pixDisplayWithTitle(pixs, 0, 0, "pixs", 1);
-        pixWrite("/tmp/lept/dewline/001.png", pixs, IFF_PNG);
+        pixWriteDebug("/tmp/lept/dewline/001.png", pixs, IFF_PNG);
     }
 
         /* Extract and solidify the horizontal and vertical lines.  We use
@@ -1688,7 +1688,7 @@ PTAA    *ptaa1, *ptaa2;
             pix1 = pixConvertTo32(pix);
             pix2 = pixDisplayPtaa(pix1, ptaa1);
             snprintf(buf, sizeof(buf), "/tmp/lept/dewline/%03d.png", 2 + 2 * i);
-            pixWrite(buf, pix2, IFF_PNG);
+            pixWriteDebug(buf, pix2, IFF_PNG);
             pixDestroy(&pix1);
             pixDestroy(&pix2);
         }
@@ -1700,7 +1700,7 @@ PTAA    *ptaa1, *ptaa2;
             pix1 = pixConvertTo32(pix);
             pix2 = pixDisplayPtaa(pix1, ptaa2);
             snprintf(buf, sizeof(buf), "/tmp/lept/dewline/%03d.png", 3 + 2 * i);
-            pixWrite(buf, pix2, IFF_PNG);
+            pixWriteDebug(buf, pix2, IFF_PNG);
             pixDestroy(&pix1);
             pixDestroy(&pix2);
         }
@@ -1750,13 +1750,13 @@ PTAA    *ptaa1, *ptaa2;
         if (dew->vsuccess == 1) {
             dewarpPopulateFullRes(dew, NULL, 0, 0);
             pix1 = fpixRenderContours(dew->fullvdispar, 3.0, 0.15);
-            pixWrite("/tmp/lept/dewline/006.png", pix1, IFF_PNG);
+            pixWriteDebug("/tmp/lept/dewline/006.png", pix1, IFF_PNG);
             pixDisplay(pix1, 1000, 0);
             pixDestroy(&pix1);
         }
         if (dew->hsuccess == 1) {
             pix1 = fpixRenderContours(dew->fullhdispar, 3.0, 0.15);
-            pixWrite("/tmp/lept/dewline/007.png", pix1, IFF_PNG);
+            pixWriteDebug("/tmp/lept/dewline/007.png", pix1, IFF_PNG);
             pixDisplay(pix1, 1000, 0);
             pixDestroy(&pix1);
         }
@@ -1786,7 +1786,7 @@ PTAA    *ptaa1, *ptaa2;
  *      (1) This tests if a model has been built, not if it is valid.
  * </pre>
  */
-l_int32
+l_ok
 dewarpaModelStatus(L_DEWARPA  *dewa,
                    l_int32     pageno,
                    l_int32    *pvsuccess,
