@@ -61,8 +61,12 @@
  *           PIX     *pixModifyBrightness()
  *
  *      Color shifting
+ *           PIX     *pixMosaicColorShiftRGB()
  *           PIX     *pixColorShiftRGB()
  *
+ *      Darken gray (unsaturated) pixels
+ *           PIX     *pixDarkenGray()
+ *
  *      General multiplicative constant color transform
  *           PIX     *pixMultConstantColor()
  *           PIX     *pixMultMatrixColor()
@@ -779,9 +783,8 @@ pixTRCMap(PIX   *pixs,
           NUMA  *na)
 {
 l_int32    w, h, d, wm, hm, wpl, wplm, i, j, sval8, dval8;
-l_int32   *tab;
 l_uint32   sval32, dval32;
-l_uint32  *data, *datam, *line, *linem;
+l_uint32  *data, *datam, *line, *linem, *tab;
 
     PROCNAME("pixTRCMap");
 
@@ -801,7 +804,7 @@ l_uint32  *data, *datam, *line, *linem;
             return ERROR_INT("pixm not 1 bpp", procName, 1);
     }
 
-    tab = numaGetIArray(na);  /* get the array for efficiency */
+    tab = (l_uint32 *)numaGetIArray(na);  /* get the array for efficiency */
     wpl = pixGetWpl(pixs);
     data = pixGetData(pixs);
     if (!pixm) {
@@ -1311,10 +1314,9 @@ PIX       *pixd;
  *
  * <pre>
  * Notes:
- *      (1) For halfwidth == 1, we implement the full sharpening filter
- *          directly.  For halfwidth == 2, we implement the the lowpass
- *          filter separably and then compute the sharpening result locally.
- *      (2) Returns a clone if no sharpening is requested.
+ *      (1) This is for %halfwidth == 1, 2.
+ *      (2) The lowpass filter is implemented separably.
+ *      (3) Returns a clone if no sharpening is requested.
  * </pre>
  */
 PIX *
@@ -1323,8 +1325,8 @@ pixUnsharpMaskingGray2D(PIX       *pixs,
                         l_float32  fract)
 {
 l_int32     w, h, d, wpls, wpld, wplf, i, j, ival, sval;
-l_uint32   *datas, *datad, *lines, *lines0, *lines1, *lines2, *lined;
-l_float32   val, norm, a[9];
+l_uint32   *datas, *datad, *lines, *lined;
+l_float32   val, norm;
 l_float32  *dataf, *linef, *linef0, *linef1, *linef2, *linef3, *linef4;
 PIX        *pixd;
 FPIX       *fpix;
@@ -1343,61 +1345,45 @@ FPIX       *fpix;
     if (halfwidth != 1 && halfwidth != 2)
         return (PIX *)ERROR_PTR("halfwidth must be 1 or 2", procName, NULL);
 
-    pixd = pixCopyBorder(NULL, pixs, halfwidth, halfwidth,
-                         halfwidth, halfwidth);
+    if ((pixd = pixCopyBorder(NULL, pixs, halfwidth, halfwidth,
+                              halfwidth, halfwidth)) == NULL)
+        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
     datad = pixGetData(pixd);
     wpld = pixGetWpl(pixd);
     datas = pixGetData(pixs);
     wpls = pixGetWpl(pixs);
 
-    if (halfwidth == 1) {
-        for (i = 0; i < 9; i++)
-            a[i] = -fract / 9.0;
-        a[4] = 1.0 + fract * 8.0 / 9.0;
-        for (i = 1; i < h - 1; i++) {
-            lines0 = datas + (i - 1) * wpls;
-            lines1 = datas + i * wpls;
-            lines2 = datas + (i + 1) * wpls;
-            lined = datad + i * wpld;
-            for (j = 1; j < w - 1; j++) {
-                val = a[0] * GET_DATA_BYTE(lines0, j - 1) +
-                      a[1] * GET_DATA_BYTE(lines0, j) +
-                      a[2] * GET_DATA_BYTE(lines0, j + 1) +
-                      a[3] * GET_DATA_BYTE(lines1, j - 1) +
-                      a[4] * GET_DATA_BYTE(lines1, j) +
-                      a[5] * GET_DATA_BYTE(lines1, j + 1) +
-                      a[6] * GET_DATA_BYTE(lines2, j - 1) +
-                      a[7] * GET_DATA_BYTE(lines2, j) +
-                      a[8] * GET_DATA_BYTE(lines2, j + 1);
-                ival = (l_int32)(val + 0.5);
-                ival = L_MAX(0, ival);
-                ival = L_MIN(255, ival);
-                SET_DATA_BYTE(lined, j, ival);
-            }
-        }
-
-        return pixd;
+        /* Do the low pass separably.  Store the result of horizontal
+         * smoothing in an intermediate fpix.  */
+    if ((fpix = fpixCreate(w, h)) == NULL) {
+        pixDestroy(&pixd);
+        return (PIX *)ERROR_PTR("fpix not made", procName, NULL);
     }
-
-        /* For halfwidth == 2, do the low pass separably.  Store
-         * the result of horizontal smoothing in an intermediate fpix.
-         * Note that in the horizontal smoothing step, the values of
-         * fpix in the 2 pixel-wide border region are unchanged from 0.
-         * In vertical smoothing, the normalization constant takes this
-         * into account for pixels near the upper and lower boundaries.  */
-    fpix = fpixCreate(w, h);
     dataf = fpixGetData(fpix);
     wplf = fpixGetWpl(fpix);
-    for (i = 2; i < h - 2; i++) {
-        lines = datas + i * wpls;
-        linef = dataf + i * wplf;
-        for (j = 2; j < w - 2; j++) {
-            val = GET_DATA_BYTE(lines, j - 2) +
-                  GET_DATA_BYTE(lines, j - 1) +
-                  GET_DATA_BYTE(lines, j) +
-                  GET_DATA_BYTE(lines, j + 1) +
-                  GET_DATA_BYTE(lines, j + 2);
-            linef[j] = val;
+    if (halfwidth == 1) {
+        for (i = 0; i < h; i++) {
+            lines = datas + i * wpls;
+            linef = dataf + i * wplf;
+            for (j = 1; j < w - 1; j++) {
+                val = GET_DATA_BYTE(lines, j - 1) +
+                      GET_DATA_BYTE(lines, j) +
+                      GET_DATA_BYTE(lines, j + 1);
+                linef[j] = val;
+            }
+        }
+    } else {
+        for (i = 0; i < h; i++) {
+            lines = datas + i * wpls;
+            linef = dataf + i * wplf;
+            for (j = 2; j < w - 2; j++) {
+                val = GET_DATA_BYTE(lines, j - 2) +
+                      GET_DATA_BYTE(lines, j - 1) +
+                      GET_DATA_BYTE(lines, j) +
+                      GET_DATA_BYTE(lines, j + 1) +
+                      GET_DATA_BYTE(lines, j + 2);
+                linef[j] = val;
+            }
         }
     }
 
@@ -1407,25 +1393,44 @@ FPIX       *fpix;
          * be added to I, then the highpass filter value is
          *     H = I - L
          * and the new sharpened value is
-         *     N = I + f * H.
-         */
-    for (i = 2; i < h - 2; i++) {
-        linef0 = dataf + (i - 2) * wplf;
-        linef1 = dataf + (i - 1) * wplf;
-        linef2 = dataf + i * wplf;
-        linef3 = dataf + (i + 1) * wplf;
-        linef4 = dataf + (i + 2) * wplf;
-        lined = datad + i * wpld;
-        lines = datas + i * wpls;
-        norm = 1.0 / (5.0 * L_MIN(L_MIN(5, i + 1), h - i));
-        for (j = 2; j < w - 2; j++) {
-            val = norm * (linef0[j] + linef1[j] + linef2[j] +
-                          linef3[j] + linef4[j]);  /* L: lowpass filter value */
-            sval = GET_DATA_BYTE(lines, j);   /* I: source pixel */
-            ival = (l_int32)(sval + fract * (sval - val) + 0.5);
-            ival = L_MAX(0, ival);
-            ival = L_MIN(255, ival);
-            SET_DATA_BYTE(lined, j, ival);
+         *     N = I + f * H.                 */
+    if (halfwidth == 1) {
+        for (i = 1; i < h - 1; i++) {
+            linef0 = dataf + (i - 1) * wplf;
+            linef1 = dataf + i * wplf;
+            linef2 = dataf + (i + 1) * wplf;
+            lined = datad + i * wpld;
+            lines = datas + i * wpls;
+            norm = 1.0 / 9.0;
+            for (j = 1; j < w - 1; j++) {
+                val = norm * (linef0[j] + linef1[j] +
+                              linef2[j]);         /* L: lowpass filter value */
+                sval = GET_DATA_BYTE(lines, j);   /* I: source pixel */
+                ival = (l_int32)(sval + fract * (sval - val) + 0.5);
+                ival = L_MAX(0, ival);
+                ival = L_MIN(255, ival);
+                SET_DATA_BYTE(lined, j, ival);
+            }
+        }
+    } else {
+        for (i = 2; i < h - 2; i++) {
+            linef0 = dataf + (i - 2) * wplf;
+            linef1 = dataf + (i - 1) * wplf;
+            linef2 = dataf + i * wplf;
+            linef3 = dataf + (i + 1) * wplf;
+            linef4 = dataf + (i + 2) * wplf;
+            lined = datad + i * wpld;
+            lines = datas + i * wpls;
+            norm = 1.0 / 25.0;
+            for (j = 2; j < w - 2; j++) {
+                val = norm * (linef0[j] + linef1[j] + linef2[j] + linef3[j] +
+                              linef4[j]);  /* L: lowpass filter value */
+                sval = GET_DATA_BYTE(lines, j);   /* I: source pixel */
+                ival = (l_int32)(sval + fract * (sval - val) + 0.5);
+                ival = L_MAX(0, ival);
+                ival = L_MIN(255, ival);
+                SET_DATA_BYTE(lined, j, ival);
+            }
         }
     }
 
@@ -1434,7 +1439,6 @@ FPIX       *fpix;
 }
 
 
-
 /*-----------------------------------------------------------------------*
  *                    Hue and saturation modification                    *
  *-----------------------------------------------------------------------*/
@@ -1699,6 +1703,106 @@ l_uint32  *data, *line;
 /*-----------------------------------------------------------------------*
  *                             Color shifting                            *
  *-----------------------------------------------------------------------*/
+/*!
+ * \brief   pixMosaicColorShiftRGB()
+ *
+ * \param[in]    pixs     32 bpp rgb
+ * \param[in]    roff   center offset of red component
+ * \param[in]    goff   center offset of green component
+ * \param[in]    boff   center offset of blue component
+ * \param[in]    delta  increments from center offsets [0.0 - 0.1];
+ *                      use 0.0 to get the default (0.04)
+ * \param[in]    nincr  number of increments in each (positive and negative)
+ *                      direction; use 0 to get the default (2).
+ * \return  pix, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) This generates a mosaic view of the effect of shifting the RGB
+ *          components.  See pixColorShiftRGB() for details on the shifting.
+ *      (2) The offsets (%roff, %goff, %boff) set the color center point,
+ *          and the deviations from this are shown separately for deltas
+ *          in r, g and b.  For each component, we show 2 * %nincr + 1
+ *          images.
+ *      (3) Usage: color prints differ from the original due to three factors:
+ *          illumination, calibration of the camera in acquisition,
+ *          and calibration of the printer.  This function can be used
+ *          to iteratively match a color print to the original.  On each
+ *          iteration, the center offsets are set to the best match so
+ *          far, and the %delta increments are typically reduced.
+ * </pre>
+ */
+PIX *
+pixMosaicColorShiftRGB(PIX       *pixs,
+                       l_float32  roff,
+                       l_float32  goff,
+                       l_float32  boff,
+                       l_float32  delta,
+                       l_int32    nincr)
+{
+char       buf[64];
+l_int32    i;
+l_float32  del;
+L_BMF     *bmf;
+PIX       *pix1, *pix2, *pix3;
+PIXA      *pixa;
+
+    PROCNAME("pixMosaicColorShiftRGB");
+
+    if (!pixs  || pixGetDepth(pixs) != 32)
+        return (PIX *)ERROR_PTR("pixs undefined or not rgb", procName, NULL);
+    if (roff < -1.0 || roff > 1.0)
+        return (PIX *)ERROR_PTR("roff not in [-1.0, 1.0]", procName, NULL);
+    if (goff < -1.0 || goff > 1.0)
+        return (PIX *)ERROR_PTR("goff not in [-1.0, 1.0]", procName, NULL);
+    if (boff < -1.0 || boff > 1.0)
+        return (PIX *)ERROR_PTR("boff not in [-1.0, 1.0]", procName, NULL);
+    if (delta < 0.0 || delta > 0.1)
+        return (PIX *)ERROR_PTR("delta not in [0.0, 0.1]", procName, NULL);
+    if (delta == 0.0) delta = 0.04;
+    if (nincr < 0 || nincr > 6)
+        return (PIX *)ERROR_PTR("nincr not in [0, 6]", procName, NULL);
+    if (nincr == 0) nincr = 2;
+
+    pixa = pixaCreate(3 * (2 * nincr + 1));
+    bmf = bmfCreate(NULL, 8);
+    pix1 = pixScaleToSize(pixs, 400, 0);
+    for (i = 0, del = - nincr * delta; i < 2 * nincr + 1; i++, del += delta) {
+        pix2 = pixColorShiftRGB(pix1, roff + del, goff, boff);
+        snprintf(buf, sizeof(buf), "%4.2f, %4.2f, %4.2f",
+                 roff + del, goff, boff);
+        pix3 = pixAddSingleTextblock(pix2, bmf, buf, 0xff000000,
+                                     L_ADD_BELOW, 0);
+        pixaAddPix(pixa, pix3, L_INSERT);
+        pixDestroy(&pix2);
+    }
+    for (i = 0, del = - nincr * delta; i < 2 * nincr + 1; i++, del += delta) {
+        pix2 = pixColorShiftRGB(pix1, roff, goff + del, boff);
+        snprintf(buf, sizeof(buf), "%4.2f, %4.2f, %4.2f",
+                 roff, goff + del, boff);
+        pix3 = pixAddSingleTextblock(pix2, bmf, buf, 0xff000000,
+                                     L_ADD_BELOW, 0);
+        pixaAddPix(pixa, pix3, L_INSERT);
+        pixDestroy(&pix2);
+    }
+    for (i = 0, del = - nincr * delta; i < 2 * nincr + 1; i++, del += delta) {
+        pix2 = pixColorShiftRGB(pix1, roff, goff, boff + del);
+        snprintf(buf, sizeof(buf), "%4.2f, %4.2f, %4.2f",
+                 roff, goff, boff + del);
+        pix3 = pixAddSingleTextblock(pix2, bmf, buf, 0xff000000,
+                                     L_ADD_BELOW, 0);
+        pixaAddPix(pixa, pix3, L_INSERT);
+        pixDestroy(&pix2);
+    }
+    pixDestroy(&pix1);
+
+    pix1 = pixaDisplayTiledAndScaled(pixa, 32, 300, 2 * nincr + 1, 0, 30, 2);
+    pixaDestroy(&pixa);
+    bmfDestroy(&bmf);
+    return pix1;
+}
+
+
 /*!
  * \brief   pixColorShiftRGB()
  *
@@ -1743,11 +1847,11 @@ PIX       *pixd;
     if (pixGetDepth(pixs) != 32)
         return (PIX *)ERROR_PTR("pixs not 32 bpp", procName, NULL);
     if (rfract < -1.0 || rfract > 1.0)
-        return (PIX *)ERROR_PTR("rfract not in [-1.0,...,1.0]", procName, NULL);
+        return (PIX *)ERROR_PTR("rfract not in [-1.0, 1.0]", procName, NULL);
     if (gfract < -1.0 || gfract > 1.0)
-        return (PIX *)ERROR_PTR("gfract not in [-1.0,...,1.0]", procName, NULL);
+        return (PIX *)ERROR_PTR("gfract not in [-1.0, 1.0]", procName, NULL);
     if (bfract < -1.0 || bfract > 1.0)
-        return (PIX *)ERROR_PTR("bfract not in [-1.0,...,1.0]", procName, NULL);
+        return (PIX *)ERROR_PTR("bfract not in [-1.0, 1.0]", procName, NULL);
     if (rfract == 0.0 && gfract == 0.0 && bfract == 0.0)
         return pixCopy(NULL, pixs);
 
@@ -1794,6 +1898,83 @@ PIX       *pixd;
     return pixd;
 }
 
+/*-----------------------------------------------------------------------*
+ *                     Darken gray (unsaturated) pixels
+ *-----------------------------------------------------------------------*/
+/*!
+ * \brief   pixDarkenGray()
+ *
+ * \param[in]    pixd      [optional] can be null or equal to pixs
+ * \param[in]    pixs      32 bpp rgb
+ * \param[in]    thresh    pixels with max component >= %thresh are unchanged
+ * \param[in]    satlimit  pixels with saturation >= %satlimit are unchanged
+ * \return  pixd, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) This darkens gray pixels, by a fraction (sat/%satlimit), where
+ *          the saturation, sat, is the component difference (max - min).
+ *          The pixel value is unchanged if sat >= %satlimit.  A typical
+ *          value of %satlimit might be 40; the larger the value, the
+ *          more that pixels with a smaller saturation will be darkened.
+ *      (2) Pixels with max component >= %thresh are unchanged. This can be
+ *          used to prevent bright pixels with low saturation from being
+ *          darkened.  Setting thresh == 0 is a no-op; setting %thresh == 255
+ *          causes the darkening to be applied to all pixels.
+ *      (3) This function is useful to enhance pixels relative to a
+ *          gray background.
+ * </pre>
+ */
+PIX *
+pixDarkenGray(PIX     *pixd,
+              PIX     *pixs,
+              l_int32  thresh,
+              l_int32  satlimit)
+{
+l_int32    w, h, i, j, wpls, wpld;
+l_int32    rval, gval, bval, minrg, min, maxrg, max, sat;
+l_uint32  *datas, *datad, *lines, *lined;
+l_float32  ratio;
+
+    PROCNAME("pixDarkenGray");
+
+    if (!pixs || pixGetDepth(pixs) != 32)
+        return (PIX *)ERROR_PTR("pixs undefined or not 32 bpp", procName, NULL);
+    if (thresh < 0 || thresh > 255)
+        return (PIX *)ERROR_PTR("invalid thresh", procName, NULL);
+    if (satlimit < 1)
+        return (PIX *)ERROR_PTR("invalid satlimit", procName, NULL);
+    if (pixd && (pixs != pixd))
+        return (PIX *)ERROR_PTR("not new or in-place", procName, NULL);
+
+    pixGetDimensions(pixs, &w, &h, NULL);
+    datas = pixGetData(pixs);
+    wpls = pixGetWpl(pixs);
+    if ((pixd = pixCopy(pixd, pixs)) == NULL)
+        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
+    datad = pixGetData(pixd);
+    wpld = pixGetWpl(pixd);
+
+    for (i = 0; i < h; i++) {
+        lines = datas + i * wpls;
+        lined = datad + i * wpld;
+        for (j = 0; j < w; j++) {
+            extractRGBValues(lines[j], &rval, &gval, &bval);
+            minrg = L_MIN(rval, gval);
+            min = L_MIN(minrg, bval);
+            maxrg = L_MAX(rval, gval);
+            max = L_MAX(maxrg, bval);
+            sat = max - min;
+            if (max >= thresh || sat >= satlimit)
+                continue;
+            ratio = (l_float32)sat / (l_float32)satlimit;
+            composeRGBPixel((l_int32)(ratio * rval), (l_int32)(ratio * gval),
+                            (l_int32)(ratio * bval), &lined[j]);
+        }
+    }
+    return pixd;
+}
+
 
 /*-----------------------------------------------------------------------*
  *            General multiplicative constant color transform            *
