@@ -430,7 +430,7 @@ pixaMorphSequenceByRegion(PIX         *pixs,
                           l_int32      minw,
                           l_int32      minh)
 {
-l_int32  n, i, w, h, samedepth, maxdepth, fullpa, fullba;
+l_int32  n, i, w, h, same, maxd, fullpa, fullba;
 BOX     *box;
 PIX     *pix1, *pix2, *pix3;
 PIXA    *pixad;
@@ -445,8 +445,8 @@ PIXA    *pixad;
         return (PIXA *)ERROR_PTR("sequence not defined", procName, NULL);
     if (!pixam)
         return (PIXA *)ERROR_PTR("pixam not defined", procName, NULL);
-    samedepth = pixaVerifyDepth(pixam, &maxdepth);
-    if (samedepth != 1 && maxdepth != 1)
+    pixaVerifyDepth(pixam, &same, &maxd);
+    if (maxd != 1)
         return (PIXA *)ERROR_PTR("mask depth not 1 bpp", procName, NULL);
     pixaIsFull(pixam, &fullpa, &fullba);
     if (!fullpa || !fullba)
@@ -680,7 +680,7 @@ PIXA    *pixa;
  *        of the Sel used to align pixp with pixs.
  * </pre>
  */
-l_int32
+l_ok
 pixRemoveMatchedPattern(PIX     *pixs,
                         PIX     *pixp,
                         PIX     *pixe,
@@ -1010,7 +1010,7 @@ PIXA      *pixad;
             pix2 = pixScaleToSize(pix1, scalew, scaleh);
             pixaAddPix(pixad, pix2, L_INSERT);
         }
-        pixDestroy(&pix1); 
+        pixDestroy(&pix1);
     }
     return pixad;
 }
@@ -1519,11 +1519,12 @@ PTA       *pta;
  *
  * <pre>
  * Notes:
- *      (1) Any table not passed in will be made internally and destroyed
+ *      (1) The sum and centroid tables are only used for 1 bpp.
+ *      (2) Any table not passed in will be made internally and destroyed
  *          after use.
  * </pre>
  */
-l_int32
+l_ok
 pixCentroid(PIX        *pix,
             l_int32    *centtab,
             l_int32    *sumtab,
@@ -1548,14 +1549,15 @@ l_int32   *ctab, *stab;
     if (d != 1 && d != 8)
         return ERROR_INT("pix not 1 or 8 bpp", procName, 1);
 
-    if (!centtab)
-        ctab = makePixelCentroidTab8();
-    else
-        ctab = centtab;
-    if (!sumtab)
-        stab = makePixelSumTab8();
-    else
-        stab = sumtab;
+    ctab = centtab;
+    stab = sumtab;
+    if (d == 1) {
+        pixSetPadBits(pix, 0);
+        if (!centtab)
+            ctab = makePixelCentroidTab8();
+        if (!sumtab)
+            stab = makePixelSumTab8();
+    }
 
     data = pixGetData(pix);
     wpl = pixGetWpl(pix);
