@@ -54,6 +54,7 @@
  *           l_int32   pixaSetBoxa()
  *           PIX     **pixaGetPixArray()
  *           l_int32   pixaVerifyDepth()
+ *           l_int32   pixaVerifyDimensions()
  *           l_int32   pixaIsFull()
  *           l_int32   pixaCountText()
  *           l_int32   pixaSetText()
@@ -67,6 +68,7 @@
  *           l_int32   pixaInsertPix()
  *           l_int32   pixaRemovePix()
  *           l_int32   pixaRemovePixAndSave()
+ *           l_int32   pixaRemoveSelected()
  *           l_int32   pixaInitFull()
  *           l_int32   pixaClear()
  *
@@ -92,6 +94,7 @@
  *           BOXA     *pixaaGetBoxa()
  *           PIX      *pixaaGetPix()
  *           l_int32   pixaaVerifyDepth()
+ *           l_int32   pixaaVerifyDimensions()
  *           l_int32   pixaaIsFull()
  *
  *      Pixaa array modifiers
@@ -104,6 +107,7 @@
  *           PIXA     *pixaRead()
  *           PIXA     *pixaReadStream()
  *           PIXA     *pixaReadMem()
+ *           l_int32   pixaWriteDebug()
  *           l_int32   pixaWrite()
  *           l_int32   pixaWriteStream()
  *           l_int32   pixaWriteMem()
@@ -147,7 +151,7 @@ static l_int32 pixaExtendArray(PIXA  *pixa);
 /*!
  * \brief   pixaCreate()
  *
- * \param[in]    n  initial number of ptrs
+ * \param[in]    n    initial number of ptrs
  * \return  pixa, or NULL on error
  *
  * <pre>
@@ -182,8 +186,8 @@ PIXA  *pixa;
 /*!
  * \brief   pixaCreateFromPix()
  *
- * \param[in]    pixs  with individual components on a lattice
- * \param[in]    n   number of components
+ * \param[in]    pixs    with individual components on a lattice
+ * \param[in]    n       number of components
  * \param[in]    cellw   width of each cell
  * \param[in]    cellh   height of each cell
  * \return  pixa, or NULL on error
@@ -242,15 +246,18 @@ PIXA    *pixa;
  *
  * \param[in]    pixs
  * \param[in]    boxa
- * \param[out]   pcropwarn [optional] TRUE if the boxa extent
- *                         is larger than pixs.
+ * \param[in]    start       first box to use
+ * \param[in]    num         number of boxes; use 0 to go to the end
+ * \param[out]   pcropwarn   [optional] TRUE if the boxa extent
+ *                           is larger than pixs.
  * \return  pixad, or NULL on error
  *
  * <pre>
  * Notes:
  *      (1) This simply extracts from pixs the region corresponding to each
- *          box in the boxa.
- *      (2) The 3rd arg is optional.  If the extent of the boxa exceeds the
+ *          box in the boxa.  To extract all the regions, set both %start
+ *          and %num to 0.
+ *      (2) The 5th arg is optional.  If the extent of the boxa exceeds the
  *          size of the pixa, so that some boxes are either clipped
  *          or entirely outside the pix, a warning is returned as TRUE.
  *      (3) pixad will have only the properly clipped elements, and
@@ -260,9 +267,11 @@ PIXA    *pixa;
 PIXA *
 pixaCreateFromBoxa(PIX      *pixs,
                    BOXA     *boxa,
+                   l_int32   start,
+                   l_int32   num,
                    l_int32  *pcropwarn)
 {
-l_int32  i, n, w, h, wbox, hbox, cropwarn;
+l_int32  i, n, end, w, h, wbox, hbox, cropwarn;
 BOX     *box, *boxc;
 PIX     *pixd;
 PIXA    *pixad;
@@ -273,9 +282,12 @@ PIXA    *pixad;
         return (PIXA *)ERROR_PTR("pixs not defined", procName, NULL);
     if (!boxa)
         return (PIXA *)ERROR_PTR("boxa not defined", procName, NULL);
+    if (num < 0)
+        return (PIXA *)ERROR_PTR("num must be >= 0", procName, NULL);
 
     n = boxaGetCount(boxa);
-    if ((pixad = pixaCreate(n)) == NULL)
+    end = (num == 0) ? n - 1 : L_MIN(start + num - 1, n - 1);
+    if ((pixad = pixaCreate(end - start + 1)) == NULL)
         return (PIXA *)ERROR_PTR("pixad not made", procName, NULL);
 
     boxaGetExtent(boxa, &wbox, &hbox, NULL);
@@ -286,7 +298,7 @@ PIXA    *pixad;
     if (pcropwarn)
         *pcropwarn = cropwarn;
 
-    for (i = 0; i < n; i++) {
+    for (i = start; i <= end; i++) {
         box = boxaGetBox(boxa, i, L_COPY);
         if (cropwarn) {  /* if box is outside pixs, pixd is NULL */
             pixd = pixClipRectangle(pixs, box, &boxc);  /* may be NULL */
@@ -309,11 +321,11 @@ PIXA    *pixad;
 /*!
  * \brief   pixaSplitPix()
  *
- * \param[in]    pixs  with individual components on a lattice
- * \param[in]    nx   number of mosaic cells horizontally
- * \param[in]    ny   number of mosaic cells vertically
- * \param[in]    borderwidth  of added border on all sides
- * \param[in]    bordercolor  in our RGBA format: 0xrrggbbaa
+ * \param[in]    pixs          with individual components on a lattice
+ * \param[in]    nx            number of mosaic cells horizontally
+ * \param[in]    ny            number of mosaic cells vertically
+ * \param[in]    borderwidth   of added border on all sides
+ * \param[in]    bordercolor   in our RGBA format: 0xrrggbbaa
  * \return  pixa, or NULL on error
  *
  * <pre>
@@ -384,7 +396,7 @@ PIXA    *pixa;
 /*!
  * \brief   pixaDestroy()
  *
- * \param[in,out]  ppixa can be nulled
+ * \param[in,out]  ppixa    use ptr address so it will be nulled
  *
  * <pre>
  * Notes:
@@ -427,7 +439,7 @@ PIXA    *pixa;
  * \brief   pixaCopy()
  *
  * \param[in]    pixa
- * \param[in]    copyflag see pix.h for details:
+ * \param[in]    copyflag  see pix.h for details:
  *                 L_COPY makes a new pixa and copies each pix and each box;
  *                 L_CLONE gives a new ref-counted handle to the input pixa;
  *                 L_COPY_CLONE makes a new pixa and inserts clones of
@@ -483,11 +495,11 @@ PIXA    *pixac;
  * \brief   pixaAddPix()
  *
  * \param[in]    pixa
- * \param[in]    pix  to be added
- * \param[in]    copyflag L_INSERT, L_COPY, L_CLONE
+ * \param[in]    pix        to be added
+ * \param[in]    copyflag   L_INSERT, L_COPY, L_CLONE
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 pixaAddPix(PIXA    *pixa,
            PIX     *pix,
            l_int32  copyflag)
@@ -528,10 +540,10 @@ PIX     *pixc;
  *
  * \param[in]    pixa
  * \param[in]    box
- * \param[in]    copyflag L_INSERT, L_COPY, L_CLONE
+ * \param[in]    copyflag    L_INSERT, L_COPY, L_CLONE
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 pixaAddBox(PIXA    *pixa,
            BOX     *box,
            l_int32  copyflag)
@@ -577,6 +589,7 @@ pixaExtendArray(PIXA  *pixa)
  * \brief   pixaExtendArrayToSize()
  *
  * \param[in]    pixa
+ * \param[in]    size
  * \return  0 if OK; 1 on error
  *
  * <pre>
@@ -585,7 +598,7 @@ pixaExtendArray(PIXA  *pixa)
  *          The pixa and boxa ptr arrays must always be equal in size.
  * </pre>
  */
-l_int32
+l_ok
 pixaExtendArrayToSize(PIXA    *pixa,
                       l_int32  size)
 {
@@ -630,9 +643,10 @@ pixaGetCount(PIXA  *pixa)
  * \brief   pixaChangeRefcount()
  *
  * \param[in]    pixa
+ * \param[in]    delta
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 pixaChangeRefcount(PIXA    *pixa,
                    l_int32  delta)
 {
@@ -650,8 +664,8 @@ pixaChangeRefcount(PIXA    *pixa,
  * \brief   pixaGetPix()
  *
  * \param[in]    pixa
- * \param[in]    index  to the index-th pix
- * \param[in]    accesstype  L_COPY or L_CLONE
+ * \param[in]    index        to the index-th pix
+ * \param[in]    accesstype   L_COPY or L_CLONE
  * \return  pix, or NULL on error
  */
 PIX *
@@ -685,11 +699,11 @@ PIX  *pix;
  * \brief   pixaGetPixDimensions()
  *
  * \param[in]    pixa
- * \param[in]    index  to the index-th box
- * \param[out]   pw, ph, pd [optional]  each can be null
+ * \param[in]    index         to the index-th box
+ * \param[out]   pw, ph, pd    [optional] each can be null
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 pixaGetPixDimensions(PIXA     *pixa,
                      l_int32   index,
                      l_int32  *pw,
@@ -720,7 +734,7 @@ PIX  *pix;
  * \brief   pixaGetBoxa()
  *
  * \param[in]    pixa
- * \param[in]    accesstype  L_COPY, L_CLONE, L_COPY_CLONE
+ * \param[in]    accesstype   L_COPY, L_CLONE, L_COPY_CLONE
  * \return  boxa, or NULL on error
  */
 BOXA *
@@ -763,8 +777,8 @@ pixaGetBoxaCount(PIXA  *pixa)
  * \brief   pixaGetBox()
  *
  * \param[in]    pixa
- * \param[in]    index  to the index-th pix
- * \param[in]    accesstype  L_COPY or L_CLONE
+ * \param[in]    index        to the index-th pix
+ * \param[in]    accesstype   L_COPY or L_CLONE
  * \return  box if null, not automatically an error, or NULL on error
  *
  * <pre>
@@ -813,11 +827,11 @@ BOX  *box;
  * \brief   pixaGetBoxGeometry()
  *
  * \param[in]    pixa
- * \param[in]    index  to the index-th box
- * \param[out]   px, py, pw, ph [optional]  each can be null
+ * \param[in]    index            to the index-th box
+ * \param[out]   px, py, pw, ph   [optional] each can be null
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 pixaGetBoxGeometry(PIXA     *pixa,
                    l_int32   index,
                    l_int32  *px,
@@ -851,7 +865,7 @@ BOX  *box;
  *
  * \param[in]    pixa
  * \param[in]    boxa
- * \param[in]    accesstype  L_INSERT, L_COPY, L_CLONE
+ * \param[in]    accesstype   L_INSERT, L_COPY, L_CLONE
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -859,7 +873,7 @@ BOX  *box;
  *      (1) This destroys the existing boxa in the pixa.
  * </pre>
  */
-l_int32
+l_ok
 pixaSetBoxa(PIXA    *pixa,
             BOXA    *boxa,
             l_int32  accesstype)
@@ -914,41 +928,95 @@ pixaGetPixArray(PIXA  *pixa)
  * \brief   pixaVerifyDepth()
  *
  * \param[in]    pixa
- * \param[out]   pmaxdepth [optional] max depth of all pix
- * \return  depth return 0 if they're not all the same, or on error
+ * \param[out]   psame   1 if depth is the same for all pix; 0 otherwise
+ * \param[out]   pmaxd   [optional] max depth of all pix
+ * \return  0 if OK, 1 on error
  *
  * <pre>
  * Notes:
  *      (1) It is considered to be an error if there are no pix.
  * </pre>
  */
-l_int32
+l_ok
 pixaVerifyDepth(PIXA     *pixa,
-                l_int32  *pmaxdepth)
+                l_int32  *psame,
+                l_int32  *pmaxd)
 {
-l_int32  i, n, d, depth, maxdepth, same;
+l_int32  i, n, d, maxd, same;
 
     PROCNAME("pixaVerifyDepth");
 
-    if (pmaxdepth) *pmaxdepth = 0;
+    if (pmaxd) *pmaxd = 0;
+    if (!psame)
+        return ERROR_INT("psame not defined", procName, 1);
     if (!pixa)
-        return ERROR_INT("pixa not defined", procName, 0);
+        return ERROR_INT("pixa not defined", procName, 1);
+    if ((n = pixaGetCount(pixa)) == 0)
+        return ERROR_INT("no pix in pixa", procName, 1);
 
-    depth = 0;
-    n = pixaGetCount(pixa);
-    maxdepth = 0;
     same = 1;
-    for (i = 0; i < n; i++) {
+    pixaGetPixDimensions(pixa, 0, NULL, NULL, &maxd);
+    for (i = 1; i < n; i++) {
         if (pixaGetPixDimensions(pixa, i, NULL, NULL, &d))
-            return ERROR_INT("pix depth not found", procName, 0);
-        maxdepth = L_MAX(maxdepth, d);
-        if (i == 0)
-            depth = d;
-        else if (d != depth)
+            return ERROR_INT("pix depth not found", procName, 1);
+        maxd = L_MAX(maxd, d);
+        if (d != maxd)
             same = 0;
     }
-    if (pmaxdepth) *pmaxdepth = maxdepth;
-    return (same == 1) ? depth : 0;
+    *psame = same;
+    if (pmaxd) *pmaxd = maxd;
+    return 0;
+}
+
+
+/*!
+ * \brief   pixaVerifyDimensions()
+ *
+ * \param[in]    pixa
+ * \param[out]   psame   1 if dimensions are the same for all pix; 0 otherwise
+ * \param[out]   pmaxw   [optional] max width of all pix
+ * \param[out]   pmaxh   [optional] max height of all pix
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) It is considered to be an error if there are no pix.
+ * </pre>
+ */
+l_ok
+pixaVerifyDimensions(PIXA     *pixa,
+                     l_int32  *psame,
+                     l_int32  *pmaxw,
+                     l_int32  *pmaxh)
+{
+l_int32  i, n, w, h, maxw, maxh, same;
+
+    PROCNAME("pixaVerifyDimensions");
+
+    if (pmaxw) *pmaxw = 0;
+    if (pmaxh) *pmaxh = 0;
+    if (!psame)
+        return ERROR_INT("psame not defined", procName, 1);
+    *psame = 0;
+    if (!pixa)
+        return ERROR_INT("pixa not defined", procName, 1);
+    if ((n = pixaGetCount(pixa)) == 0)
+        return ERROR_INT("no pix in pixa", procName, 1);
+
+    same = 1;
+    pixaGetPixDimensions(pixa, 0, &maxw, &maxh, NULL);
+    for (i = 1; i < n; i++) {
+        if (pixaGetPixDimensions(pixa, i, &w, &h, NULL))
+            return ERROR_INT("pix dimensions not found", procName, 1);
+        maxw = L_MAX(maxw, w);
+        maxh = L_MAX(maxh, h);
+        if (w != maxw || h != maxh)
+            same = 0;
+    }
+    *psame = same;
+    if (pmaxw) *pmaxw = maxw;
+    if (pmaxh) *pmaxh = maxh;
+    return 0;
 }
 
 
@@ -956,8 +1024,8 @@ l_int32  i, n, d, depth, maxdepth, same;
  * \brief   pixaIsFull()
  *
  * \param[in]    pixa
- * \param[out]   pfullpa [optional] 1 if pixa is full
- * \param[out]   pfullba [optional] 1 if boxa is full
+ * \param[out]   pfullpa   [optional] 1 if pixa is full
+ * \param[out]   pfullba   [optional] 1 if boxa is full
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -966,7 +1034,7 @@ l_int32  i, n, d, depth, maxdepth, same;
  *          occupied from index 0 to index (pixa->n - 1).
  * </pre>
  */
-l_int32
+l_ok
 pixaIsFull(PIXA     *pixa,
            l_int32  *pfullpa,
            l_int32  *pfullba)
@@ -1007,7 +1075,7 @@ PIX     *pix;
  * \brief   pixaCountText()
  *
  * \param[in]    pixa
- * \param[out]   pntext number of pix with non-empty text strings
+ * \param[out]   pntext    number of pix with non-empty text strings
  * \return  0 if OK, 1 on error.
  *
  * <pre>
@@ -1016,7 +1084,7 @@ PIX     *pix;
  *          equals the pixa count.
  * </pre>
  */
-l_int32
+l_ok
 pixaCountText(PIXA     *pixa,
               l_int32  *pntext)
 {
@@ -1050,18 +1118,22 @@ PIX     *pix;
  * \brief   pixaSetText()
  *
  * \param[in]    pixa
- * \param[in]    sa  [optional] array of text strings, to insert in each pix
+ * \param[in]    text  [optional] single text string, to insert in each pix
+ * \param[in]    sa    [optional] array of text strings, to insert in each pix
  * \return  0 if OK, 1 on error.
  *
  * <pre>
  * Notes:
- *      (1) To clear all the text fields, use sa == NULL;
- *      (2) If sa is defined, it must be the same size as %pixa.
+ *      (1) To clear all the text fields, use %sa == NULL and %text == NULL.
+ *      (2) To set all the text fields to the same value %text, use %sa = NULL.
+ *      (3) If %sa is defined, we ignore %text and use it; %sa must have
+ *          the same count as %pixa.
  * </pre>
  */
-l_int32
-pixaSetText(PIXA    *pixa,
-            SARRAY  *sa)
+l_ok
+pixaSetText(PIXA        *pixa,
+            const char  *text,
+            SARRAY      *sa)
 {
 char    *str;
 l_int32  i, n;
@@ -1073,19 +1145,19 @@ PIX     *pix;
         return ERROR_INT("pixa not defined", procName, 1);
 
     n = pixaGetCount(pixa);
+    if (sa && (sarrayGetCount(sa) != n))
+        return ERROR_INT("pixa and sa sizes differ", procName, 1);
+
     if (!sa) {
         for (i = 0; i < n; i++) {
             if ((pix = pixaGetPix(pixa, i, L_CLONE)) == NULL)
                 continue;
-            pixSetText(pix, NULL);
+            pixSetText(pix, text);
             pixDestroy(&pix);
         }
         return 0;
     }
 
-    if (sarrayGetCount(sa) != n)
-        return ERROR_INT("pixa and sa sizes differ", procName, 1);
-
     for (i = 0; i < n; i++) {
         if ((pix = pixaGetPix(pixa, i, L_CLONE)) == NULL)
             continue;
@@ -1101,8 +1173,8 @@ PIX     *pix;
 /*!
  * \brief   pixaGetLinePtrs()
  *
- * \param[in]    pixa of pix that all have the same depth
- * \param[out]   psize [optional] number of pix in the pixa
+ * \param[in]    pixa    of pix that all have the same depth
+ * \param[out]   psize   [optional] number of pix in the pixa
  * \return  array of array of line ptrs, or NULL on error
  *
  * <pre>
@@ -1121,7 +1193,7 @@ void ***
 pixaGetLinePtrs(PIXA     *pixa,
                 l_int32  *psize)
 {
-l_int32  i, n;
+l_int32  i, n, same;
 void   **lineptrs;
 void  ***lineset;
 PIX     *pix;
@@ -1131,7 +1203,8 @@ PIX     *pix;
     if (psize) *psize = 0;
     if (!pixa)
         return (void ***)ERROR_PTR("pixa not defined", procName, NULL);
-    if (pixaVerifyDepth(pixa, NULL) == 0)
+    pixaVerifyDepth(pixa, &same, NULL);
+    if (!same)
         return (void ***)ERROR_PTR("pixa not all same depth", procName, NULL);
     n = pixaGetCount(pixa);
     if (psize) *psize = n;
@@ -1154,7 +1227,7 @@ PIX     *pix;
 /*!
  * \brief   pixaWriteStreamInfo()
  *
- * \param[in]    fp file stream
+ * \param[in]    fp     file stream
  * \param[in]    pixa
  * \return  0 if OK, 1 on error.
  *
@@ -1164,7 +1237,7 @@ PIX     *pix;
  *          text string (if it exists), and cmap info.
  * </pre>
  */
-l_int32
+l_ok
 pixaWriteStreamInfo(FILE  *fp,
                     PIXA  *pixa)
 {
@@ -1211,9 +1284,9 @@ PIXCMAP  *cmap;
  * \brief   pixaReplacePix()
  *
  * \param[in]    pixa
- * \param[in]    index  to the index-th pix
- * \param[in]    pix insert to replace existing one
- * \param[in]    box [optional] insert to replace existing
+ * \param[in]    index   to the index-th pix
+ * \param[in]    pix     insert to replace existing one
+ * \param[in]    box     [optional] insert to replace existing
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1222,7 +1295,7 @@ PIXCMAP  *cmap;
  *      (2) The previous pix at that location is destroyed.
  * </pre>
  */
-l_int32
+l_ok
 pixaReplacePix(PIXA    *pixa,
                l_int32  index,
                PIX     *pix,
@@ -1257,9 +1330,9 @@ BOXA  *boxa;
  * \brief   pixaInsertPix()
  *
  * \param[in]    pixa
- * \param[in]    index at which pix is to be inserted
- * \param[in]    pixs new pix to be inserted
- * \param[in]    box [optional] new box to be inserted
+ * \param[in]    index   at which pix is to be inserted
+ * \param[in]    pixs    new pix to be inserted
+ * \param[in]    box     [optional] new box to be inserted
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1272,7 +1345,7 @@ BOXA  *boxa;
  *      (4) To append a pix to a pixa, it's easier to use pixaAddPix().
  * </pre>
  */
-l_int32
+l_ok
 pixaInsertPix(PIXA    *pixa,
               l_int32  index,
               PIX     *pixs,
@@ -1311,7 +1384,7 @@ l_int32  i, n;
  * \brief   pixaRemovePix()
  *
  * \param[in]    pixa
- * \param[in]    index of pix to be removed
+ * \param[in]    index    of pix to be removed
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1322,7 +1395,7 @@ l_int32  i, n;
  *      (3) The corresponding box is removed as well, if it exists.
  * </pre>
  */
-l_int32
+l_ok
 pixaRemovePix(PIXA    *pixa,
               l_int32  index)
 {
@@ -1360,9 +1433,9 @@ PIX    **array;
  * \brief   pixaRemovePixAndSave()
  *
  * \param[in]    pixa
- * \param[in]    index of pix to be removed
- * \param[out]   ppix [optional] removed pix
- * \param[out]   pbox [optional] removed box
+ * \param[in]    index   of pix to be removed
+ * \param[out]   ppix    [optional] removed pix
+ * \param[out]   pbox    [optional] removed box
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1374,7 +1447,7 @@ PIX    **array;
  *      (4) The removed pix and box can either be retained or destroyed.
  * </pre>
  */
-l_int32
+l_ok
 pixaRemovePixAndSave(PIXA    *pixa,
                      l_int32  index,
                      PIX    **ppix,
@@ -1414,12 +1487,52 @@ PIX    **array;
 }
 
 
+/*!
+ * \brief   pixaRemoveSelected()
+ *
+ * \param[in]    pixa
+ * \param[in]    naindex   numa of indices of pix to be removed
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) This gives error messages for invalid indices
+ * </pre>
+ */
+l_ok
+pixaRemoveSelected(PIXA  *pixa,
+                   NUMA  *naindex)
+{
+l_int32  i, n, npix, index;
+NUMA    *na1;
+
+    PROCNAME("pixaRemoveSelected");
+
+    if (!pixa)
+        return ERROR_INT("pixa not defined", procName, 1);
+    if (!naindex)
+        return ERROR_INT("naindex not defined", procName, 1);
+    if ((n = numaGetCount(naindex)) == 0)
+        return ERROR_INT("naindex is empty", procName, 1);
+    npix = pixaGetCount(pixa);
+
+        /* Remove from highest indices first */
+    na1 = numaSort(NULL, naindex, L_SORT_DECREASING);
+    for (i = 0; i < n; i++) {
+        numaGetIValue(na1, i, &index);
+        pixaRemovePix(pixa, index);
+    }
+    numaDestroy(&na1);
+    return 0;
+}
+
+
 /*!
  * \brief   pixaInitFull()
  *
- * \param[in]    pixa typically empty
- * \param[in]    pix [optional] to be replicated into the entire pixa ptr array
- * \param[in]    box [optional] to be replicated into the entire boxa ptr array
+ * \param[in]    pixa   typically empty
+ * \param[in]    pix    [optional] to be replicated to the entire pixa ptr array
+ * \param[in]    box    [optional] to be replicated to the entire boxa ptr array
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1449,7 +1562,7 @@ PIX    **array;
  *          filled, even if all pix (and boxes) are not later replaced.
  * </pre>
  */
-l_int32
+l_ok
 pixaInitFull(PIXA  *pixa,
              PIX   *pix,
              BOX   *box)
@@ -1491,7 +1604,7 @@ PIX     *pix1;
  *          are all null'd.  The number of allocated pix, n, is set to 0.
  * </pre>
  */
-l_int32
+l_ok
 pixaClear(PIXA  *pixa)
 {
 l_int32  i, n;
@@ -1515,10 +1628,10 @@ l_int32  i, n;
 /*!
  * \brief   pixaJoin()
  *
- * \param[in]    pixad  dest pixa; add to this one
- * \param[in]    pixas  [optional] source pixa; add from this one
- * \param[in]    istart  starting index in pixas
- * \param[in]    iend  ending index in pixas; use -1 to cat all
+ * \param[in]    pixad    dest pixa; add to this one
+ * \param[in]    pixas    [optional] source pixa; add from this one
+ * \param[in]    istart   starting index in pixas
+ * \param[in]    iend     ending index in pixas; use -1 to cat all
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1529,7 +1642,7 @@ l_int32  i, n;
  *      (4) If pixas is NULL or contains no pix, this is a no-op.
  * </pre>
  */
-l_int32
+l_ok
 pixaJoin(PIXA    *pixad,
          PIXA    *pixas,
          l_int32  istart,
@@ -1572,9 +1685,9 @@ PIX     *pix;
 /*!
  * \brief   pixaInterleave()
  *
- * \param[in]    pixa1  first src pixa
- * \param[in]    pixa2  second src pixa
- * \param[in]    copyflag L_CLONE, L_COPY
+ * \param[in]    pixa1      first src pixa
+ * \param[in]    pixa2      second src pixa
+ * \param[in]    copyflag   L_CLONE, L_COPY
  * \return  pixa  interleaved from sources, or NULL on error.
  *
  * <pre>
@@ -1637,10 +1750,10 @@ PIXA    *pixad;
 /*!
  * \brief   pixaaJoin()
  *
- * \param[in]    paad  dest pixaa; add to this one
- * \param[in]    paas  [optional] source pixaa; add from this one
- * \param[in]    istart  starting index in pixaas
- * \param[in]    iend  ending index in pixaas; use -1 to cat all
+ * \param[in]    paad     dest pixaa; add to this one
+ * \param[in]    paas     [optional] source pixaa; add from this one
+ * \param[in]    istart   starting index in pixaas
+ * \param[in]    iend     ending index in pixaas; use -1 to cat all
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1650,7 +1763,7 @@ PIXA    *pixad;
  *      (3) iend < 0 means 'read to the end'
  * </pre>
  */
-l_int32
+l_ok
 pixaaJoin(PIXAA   *paad,
           PIXAA   *paas,
           l_int32  istart,
@@ -1689,7 +1802,7 @@ PIXA    *pixa;
 /*!
  * \brief   pixaaCreate()
  *
- * \param[in]    n  initial number of pixa ptrs
+ * \param[in]    n    initial number of pixa ptrs
  * \return  paa, or NULL on error
  *
  * <pre>
@@ -1737,9 +1850,9 @@ PIXAA  *paa;
  * \brief   pixaaCreateFromPixa()
  *
  * \param[in]    pixa
- * \param[in]    n number specifying subdivision of pixa
- * \param[in]    type L_CHOOSE_CONSECUTIVE, L_CHOOSE_SKIP_BY
- * \param[in]    copyflag L_CLONE, L_COPY
+ * \param[in]    n          number specifying subdivision of pixa
+ * \param[in]    type       L_CHOOSE_CONSECUTIVE, L_CHOOSE_SKIP_BY
+ * \param[in]    copyflag   L_CLONE, L_COPY
  * \return  paa, or NULL on error
  *
  * <pre>
@@ -1812,7 +1925,7 @@ PIXAA   *paa;
 /*!
  * \brief   pixaaDestroy()
  *
- * \param[in,out]   ppaa to be nulled
+ * \param[in,out]   ppaa    use ptr address so it will be nulled
  * \return  void
  */
 void
@@ -1850,7 +1963,7 @@ PIXAA   *paa;
  * \brief   pixaaAddPixa()
  *
  * \param[in]    paa
- * \param[in]    pixa  to be added
+ * \param[in]    pixa    to be added
  * \param[in]    copyflag:
  *                 L_INSERT inserts the pixa directly;
  *                 L_COPY makes a new pixa and copies each pix and each box;
@@ -1859,7 +1972,7 @@ PIXAA   *paa;
  *                     all pix and boxes
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 pixaaAddPixa(PIXAA   *paa,
              PIXA    *pixa,
              l_int32  copyflag)
@@ -1900,7 +2013,7 @@ PIXA    *pixac;
  * \param[in]    paa
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 pixaaExtendArray(PIXAA  *paa)
 {
     PROCNAME("pixaaExtendArray");
@@ -1921,14 +2034,14 @@ pixaaExtendArray(PIXAA  *paa)
 /*!
  * \brief   pixaaAddPix()
  *
- * \param[in]    paa  input paa
- * \param[in]    index index of pixa in paa
- * \param[in]    pix to be added
- * \param[in]    box [optional] to be added
- * \param[in]    copyflag L_INSERT, L_COPY, L_CLONE
+ * \param[in]    paa        input paa
+ * \param[in]    index      index of pixa in paa
+ * \param[in]    pix        to be added
+ * \param[in]    box        [optional] to be added
+ * \param[in]    copyflag   L_INSERT, L_COPY, L_CLONE
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 pixaaAddPix(PIXAA   *paa,
             l_int32  index,
             PIX     *pix,
@@ -1958,7 +2071,7 @@ PIXA  *pixa;
  *
  * \param[in]    paa
  * \param[in]    box
- * \param[in]    copyflag L_INSERT, L_COPY, L_CLONE
+ * \param[in]    copyflag    L_INSERT, L_COPY, L_CLONE
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -1967,7 +2080,7 @@ PIXA  *pixa;
  *          of a pixa that is being added to the pixaa.
  * </pre>
  */
-l_int32
+l_ok
 pixaaAddBox(PIXAA   *paa,
             BOX     *box,
             l_int32  copyflag)
@@ -1994,7 +2107,7 @@ pixaaAddBox(PIXAA   *paa,
  * \brief   pixaaGetCount()
  *
  * \param[in]    paa
- * \param[out]   pna [optional] number of pix in each pixa
+ * \param[out]   pna    [optional] number of pix in each pixa
  * \return  count, or 0 if no pixaa
  *
  * <pre>
@@ -2035,8 +2148,8 @@ PIXA    *pixa;
  * \brief   pixaaGetPixa()
  *
  * \param[in]    paa
- * \param[in]    index  to the index-th pixa
- * \param[in]    accesstype  L_COPY, L_CLONE, L_COPY_CLONE
+ * \param[in]    index        to the index-th pixa
+ * \param[in]    accesstype   L_COPY, L_CLONE, L_COPY_CLONE
  * \return  pixa, or NULL on error
  *
  * <pre>
@@ -2080,7 +2193,7 @@ PIXA  *pixa;
  * \brief   pixaaGetBoxa()
  *
  * \param[in]    paa
- * \param[in]    accesstype  L_COPY, L_CLONE
+ * \param[in]    accesstype    L_COPY, L_CLONE
  * \return  boxa, or NULL on error
  *
  * <pre>
@@ -2108,9 +2221,9 @@ pixaaGetBoxa(PIXAA   *paa,
  * \brief   pixaaGetPix()
  *
  * \param[in]    paa
- * \param[in]    index  index into the pixa array in the pixaa
- * \param[in]    ipix  index into the pix array in the pixa
- * \param[in]    accessflag  L_COPY or L_CLONE
+ * \param[in]    index        index into the pixa array in the pixaa
+ * \param[in]    ipix         index into the pix array in the pixa
+ * \param[in]    accessflag   L_COPY or L_CLONE
  * \return  pix, or NULL on error
  */
 PIX *
@@ -2137,36 +2250,104 @@ PIXA  *pixa;
  * \brief   pixaaVerifyDepth()
  *
  * \param[in]    paa
- * \param[out]   pmaxdepth [optional] max depth of all pix in pixaa
- * \return  depth return 0 if they're not all the same, or on error
+ * \param[out]   psame   1 if all pix have the same depth; 0 otherwise
+ * \param[out]   pmaxd   [optional] max depth of all pix in pixaa
+ * \return   0 if OK; 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) It is considered to be an error if any pixa have no pix.
+ * </pre>
  */
-l_int32
+l_ok
 pixaaVerifyDepth(PIXAA    *paa,
-                 l_int32  *pmaxdepth)
+                 l_int32  *psame,
+                 l_int32  *pmaxd)
 {
-l_int32  i, npixa, d, maxd, maxdepth, same;
+l_int32  i, n, d, maxd, same, samed;
 PIXA    *pixa;
 
     PROCNAME("pixaaVerifyDepth");
 
-    if (pmaxdepth) *pmaxdepth = 0;
+    if (pmaxd) *pmaxd = 0;
+    if (!psame)
+        return ERROR_INT("psame not defined", procName, 1);
+    *psame = 0;
     if (!paa)
-        return ERROR_INT("paa not defined", procName, 0);
+        return ERROR_INT("paa not defined", procName, 1);
+    if ((n = pixaaGetCount(paa, NULL)) == 0)
+        return ERROR_INT("no pixa in paa", procName, 1);
 
-    npixa = pixaaGetCount(paa, NULL);
-    maxdepth = 0;
-    same = 1;
-    for (i = 0; i < npixa; i++) {
+    pixa = pixaaGetPixa(paa, 0, L_CLONE);
+    pixaVerifyDepth(pixa, &same, &maxd);  /* init same, maxd with first pixa */
+    pixaDestroy(&pixa);
+    for (i = 1; i < n; i++) {
         pixa = pixaaGetPixa(paa, i, L_CLONE);
-        if (pixaGetCount(pixa) > 0) {
-            d = pixaVerifyDepth(pixa, &maxd);
-            maxdepth = L_MAX(maxdepth, maxd);  /* biggest up to this point */
-            if (d != maxdepth) same = 0;
-        }
+        pixaVerifyDepth(pixa, &samed, &d);
         pixaDestroy(&pixa);
+        maxd = L_MAX(maxd, d);
+        if (!samed || maxd != d)
+            same = 0;
     }
-    if (pmaxdepth) *pmaxdepth = maxdepth;
-    return (same == 1) ? maxdepth : 0;
+    *psame = same;
+    if (pmaxd) *pmaxd = maxd;
+    return 0;
+}
+
+
+/*!
+ * \brief   pixaaVerifyDimensions()
+ *
+ * \param[in]    paa
+ * \param[out]   psame   1 if all pix have the same depth; 0 otherwise
+ * \param[out]   pmaxw   [optional] max width of all pix in pixaa
+ * \param[out]   pmaxh   [optional] max height of all pix in pixaa
+ * \return   0 if OK; 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) It is considered to be an error if any pixa have no pix.
+ * </pre>
+ */
+l_ok
+pixaaVerifyDimensions(PIXAA    *paa,
+                      l_int32  *psame,
+                      l_int32  *pmaxw,
+                      l_int32  *pmaxh)
+{
+l_int32  i, n, w, h, maxw, maxh, same, same2;
+PIXA    *pixa;
+
+    PROCNAME("pixaaVerifyDimensions");
+
+    if (pmaxw) *pmaxw = 0;
+    if (pmaxh) *pmaxh = 0;
+    if (!psame)
+        return ERROR_INT("psame not defined", procName, 1);
+    *psame = 0;
+    if (!paa)
+        return ERROR_INT("paa not defined", procName, 1);
+    if ((n = pixaaGetCount(paa, NULL)) == 0)
+        return ERROR_INT("no pixa in paa", procName, 1);
+
+        /* Init same; init maxw and maxh from first pixa */
+    pixa = pixaaGetPixa(paa, 0, L_CLONE);
+    pixaVerifyDimensions(pixa, &same, &maxw, &maxh);
+    pixaDestroy(&pixa);
+
+    for (i = 1; i < n; i++) {
+        pixa = pixaaGetPixa(paa, i, L_CLONE);
+        pixaVerifyDimensions(pixa, &same2, &w, &h);
+        pixaDestroy(&pixa);
+        maxw = L_MAX(maxw, w);
+        maxh = L_MAX(maxh, h);
+        if (!same2 || maxw != w || maxh != h)
+            same = 0;
+    }
+    *psame = same;
+    if (pmaxw) *pmaxw = maxw;
+    if (pmaxh) *pmaxh = maxh;
+    return 0;
 }
 
 
@@ -2174,7 +2355,7 @@ PIXA    *pixa;
  * \brief   pixaaIsFull()
  *
  * \param[in]    paa
- * \param[out]   pfull 1 if all pixa in the paa have full pix arrays
+ * \param[out]   pfull    1 if all pixa in the paa have full pix arrays
  * \return  return 0 if OK, 1 on error
  *
  * <pre>
@@ -2216,8 +2397,8 @@ PIXA    *pixa;
 /*!
  * \brief   pixaaInitFull()
  *
- * \param[in]    paa typically empty
- * \param[in]    pixa to be replicated into the entire pixa ptr array
+ * \param[in]    paa     typically empty
+ * \param[in]    pixa    to be replicated into the entire pixa ptr array
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -2234,7 +2415,7 @@ PIXA    *pixa;
  *          The initialization allows the pixaa to always be properly filled.
  * </pre>
  */
-l_int32
+l_ok
 pixaaInitFull(PIXAA  *paa,
               PIXA   *pixa)
 {
@@ -2264,7 +2445,7 @@ PIXA    *pixat;
  *
  * \param[in]    paa
  * \param[in]    index  to the index-th pixa
- * \param[in]    pixa insert to replace existing one
+ * \param[in]    pixa   insert to replace existing one
  * \return  0 if OK, 1 on error
  *
  * <pre>
@@ -2276,7 +2457,7 @@ PIXA    *pixat;
  *      (3) The index must be within the allowed set.
  * </pre>
  */
-l_int32
+l_ok
 pixaaReplacePixa(PIXAA   *paa,
                  l_int32  index,
                  PIXA    *pixa)
@@ -2309,7 +2490,7 @@ pixaaReplacePixa(PIXAA   *paa,
  *          in the pixa ptr array.
  * </pre>
  */
-l_int32
+l_ok
 pixaaClear(PIXAA  *paa)
 {
 l_int32  i, n;
@@ -2340,7 +2521,7 @@ l_int32  i, n;
  *          and resets the count.
  * </pre>
  */
-l_int32
+l_ok
 pixaaTruncate(PIXAA  *paa)
 {
 l_int32  i, n, np;
@@ -2415,7 +2596,7 @@ PIXA  *pixa;
 /*!
  * \brief   pixaReadStream()
  *
- * \param[in]    fp file stream
+ * \param[in]    fp    file stream
  * \return  pixa, or NULL on error
  *
  * <pre>
@@ -2479,8 +2660,8 @@ PIXA    *pixa;
 /*!
  * \brief   pixaReadMem()
  *
- * \param[in]    data  of serialized pixa
- * \param[in]    size  of data in bytes
+ * \param[in]    data   of serialized pixa
+ * \param[in]    size   of data in bytes
  * \return  pixa, or NULL on error
  */
 PIXA *
@@ -2504,6 +2685,37 @@ PIXA  *pixa;
 }
 
 
+/*!
+ * \brief   pixaWriteDebug()
+ *
+ * \param[in]    fname
+ * \param[in]    pixa
+ * \return  0 if OK; 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) Debug version, intended for use in the library when writing
+ *          to files in a temp directory with names that are compiled in.
+ *          This is used instead of pixaWrite() for all such library calls.
+ *      (2) The global variable LeptDebugOK defaults to 0, and can be set
+ *          or cleared by the function setLeptDebugOK().
+ * </pre>
+ */
+l_ok
+pixaWriteDebug(const char  *fname,
+               PIXA        *pixa)
+{
+    PROCNAME("pixaWriteDebug");
+
+    if (LeptDebugOK) {
+        return pixaWrite(fname, pixa);
+    } else {
+        L_INFO("write to named temp file %s is disabled\n", procName, fname);
+        return 0;
+    }
+}
+
+
 /*!
  * \brief   pixaWrite()
  *
@@ -2517,7 +2729,7 @@ PIXA  *pixa;
  *          If the png library is not linked, this will fail.
  * </pre>
  */
-l_int32
+l_ok
 pixaWrite(const char  *filename,
           PIXA        *pixa)
 {
@@ -2548,7 +2760,7 @@ FILE    *fp;
 /*!
  * \brief   pixaWriteStream()
  *
- * \param[in]    fp file stream opened for "wb"
+ * \param[in]    fp     file stream opened for "wb"
  * \param[in]    pixa
  * \return  0 if OK, 1 on error
  *
@@ -2558,7 +2770,7 @@ FILE    *fp;
  *          If the png library is not linked, this will fail.
  * </pre>
  */
-l_int32
+l_ok
 pixaWriteStream(FILE  *fp,
                 PIXA  *pixa)
 {
@@ -2595,8 +2807,8 @@ PIX     *pix;
 /*!
  * \brief   pixaWriteMem()
  *
- * \param[out]   pdata data of serialized pixa
- * \param[out]   psize size of returned data
+ * \param[out]   pdata    data of serialized pixa
+ * \param[out]   psize    size of returned data
  * \param[in]    pixa
  * \return  0 if OK, 1 on error
  *
@@ -2605,7 +2817,7 @@ PIX     *pix;
  *      (1) Serializes a pixa in memory and puts the result in a buffer.
  * </pre>
  */
-l_int32
+l_ok
 pixaWriteMem(l_uint8  **pdata,
              size_t    *psize,
              PIXA      *pixa)
@@ -2698,10 +2910,10 @@ PIXAC  *pac;
 /*!
  * \brief   pixaaReadFromFiles()
  *
- * \param[in]    dirname directory
- * \param[in]    substr [optional] substring filter on filenames; can be NULL
- * \param[in]    first 0-based
- * \param[in]    nfiles use 0 for everything from %first to the end
+ * \param[in]    dirname   directory
+ * \param[in]    substr    [optional] substring filter on filenames; can be NULL
+ * \param[in]    first     0-based
+ * \param[in]    nfiles    use 0 for everything from %first to the end
  * \return  paa, or NULL on error or if no pixa files are found.
  *
  * <pre>
@@ -2793,7 +3005,7 @@ PIXAA  *paa;
 /*!
  * \brief   pixaaReadStream()
  *
- * \param[in]    fp file stream
+ * \param[in]    fp    file stream
  * \return  paa, or NULL on error
  *
  * <pre>
@@ -2856,8 +3068,8 @@ PIXAA   *paa;
 /*!
  * \brief   pixaaReadMem()
  *
- * \param[in]    data  of serialized pixaa
- * \param[in]    size  of data in bytes
+ * \param[in]    data   of serialized pixaa
+ * \param[in]    size   of data in bytes
  * \return  paa, or NULL on error
  */
 PIXAA *
@@ -2894,7 +3106,7 @@ PIXAA  *paa;
  *          If the png library is not linked, this will fail.
  * </pre>
  */
-l_int32
+l_ok
 pixaaWrite(const char  *filename,
            PIXAA       *paa)
 {
@@ -2925,7 +3137,7 @@ FILE    *fp;
 /*!
  * \brief   pixaaWriteStream()
  *
- * \param[in]    fp file stream opened for "wb"
+ * \param[in]    fp    file stream opened for "wb"
  * \param[in]    paa
  * \return  0 if OK, 1 on error
  *
@@ -2935,7 +3147,7 @@ FILE    *fp;
  *          If the png library is not linked, this will fail.
  * </pre>
  */
-l_int32
+l_ok
 pixaaWriteStream(FILE   *fp,
                  PIXAA  *paa)
 {
@@ -2971,8 +3183,8 @@ PIXA    *pixa;
 /*!
  * \brief   pixaaWriteMem()
  *
- * \param[out]   pdata data of serialized pixaa
- * \param[out]   psize size of returned data
+ * \param[out]   pdata   data of serialized pixaa
+ * \param[out]   psize   size of returned data
  * \param[in]    paa
  * \return  0 if OK, 1 on error
  *
@@ -2981,7 +3193,7 @@ PIXA    *pixa;
  *      (1) Serializes a pixaa in memory and puts the result in a buffer.
  * </pre>
  */
-l_int32
+l_ok
 pixaaWriteMem(l_uint8  **pdata,
               size_t    *psize,
               PIXAA     *paa)
