@@ -28,6 +28,17 @@
  * \file utils1.c
  * <pre>
  *
+ *       ------------------------------------------
+ *       This file has these utilities:
+ *         - error, warning and info messages
+ *         - low-level endian conversions
+ *         - file corruption operations
+ *         - random and prime number operations
+ *         - 64-bit hash functions
+ *         - leptonica version number accessor
+ *         - timing and date operations
+ *       ------------------------------------------
+ *
  *       Control of error, warning and info messages
  *           l_int32    setMsgSeverity()
  *
@@ -63,10 +74,10 @@
  *       Prime finders
  *           l_int32    findNextLargerPrime()
  *           l_int32    lept_isPrime()
-  *
+ *
  *       Gray code conversion
- *           l_uint32   convertBinaryToGrayCode()
- *           l_uint32   convertGrayToBinaryCode()
+ *           l_uint32   convertIntToGrayCode()
+ *           l_uint32   convertGrayCodeToInt()
  *
  *       Leptonica version number
  *           char      *getLeptonicaVersion()
@@ -93,7 +104,6 @@
 #include <windows.h>
 #endif  /* _WIN32 */
 
-//#include <string.h>
 #include <time.h>
 #include "allheaders.h"
 #include <math.h>
@@ -230,7 +240,7 @@ returnErrorPtr(const char  *msg,
  * \param[out]   psame 1 if identical; 0 if different
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 filesAreIdentical(const char  *fname1,
                   const char  *fname2,
                   l_int32     *psame)
@@ -254,8 +264,10 @@ l_uint8  *array1, *array2;
 
     if ((array1 = l_binaryRead(fname1, &nbytes1)) == NULL)
         return ERROR_INT("array1 not read", procName, 1);
-    if ((array2 = l_binaryRead(fname2, &nbytes2)) == NULL)
+    if ((array2 = l_binaryRead(fname2, &nbytes2)) == NULL) {
+        LEPT_FREE(array1);
         return ERROR_INT("array2 not read", procName, 1);
+    }
     same = 1;
     for (i = 0; i < nbytes1; i++) {
         if (array1[i] != array2[i]) {
@@ -372,7 +384,7 @@ convertOnBigEnd32(l_uint32  wordin)
  *          data is corrupted, by simulating data corruption by deletion.
  * </pre>
  */
-l_int32
+l_ok
 fileCorruptByDeletion(const char  *filein,
                       l_float32    loc,
                       l_float32    size,
@@ -436,7 +448,7 @@ l_uint8  *datain, *dataout;
  *          data is corrupted, by simulating data corruption.
  * </pre>
  */
-l_int32
+l_ok
 fileCorruptByMutation(const char  *filein,
                       l_float32    loc,
                       l_float32    size,
@@ -494,7 +506,7 @@ l_uint8  *data;
  *          use %range = 100.
  * </pre>
  */
-l_int32
+l_ok
 genRandomIntegerInRange(l_int32   range,
                         l_int32   seed,
                         l_int32  *pval)
@@ -563,7 +575,7 @@ lept_roundftoi(l_float32  fval)
  *          collisions for this set.
  * </pre>
  */
-l_int32
+l_ok
 l_hashStringToUint64(const char  *str,
                      l_uint64    *phash)
 {
@@ -609,7 +621,7 @@ l_uint64  hash, mulp;
  *          values are not required.
  * </pre>
  */
-l_int32
+l_ok
 l_hashPtToUint64(l_int32    x,
                  l_int32    y,
                  l_uint64  *phash)
@@ -649,7 +661,7 @@ l_hashPtToUint64(l_int32    x,
  *          and a dnahash hashmap are built.  See l_dnaMakeHistoByHash().
  * </pre>
  */
-l_int32
+l_ok
 l_hashFloat64ToUint64(l_int32    nbuckets,
                       l_float64  val,
                       l_uint64  *phash)
@@ -673,7 +685,7 @@ l_hashFloat64ToUint64(l_int32    nbuckets,
  * \param[out]   pprime first prime larger than %start
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 findNextLargerPrime(l_int32    start,
                     l_uint32  *pprime)
 {
@@ -708,7 +720,7 @@ l_int32  i, is_prime;
  *                       or 0 on error or if prime
  * \return  0 if OK, 1 on error
  */
-l_int32
+l_ok
 lept_isPrime(l_uint64   n,
              l_int32   *pis_prime,
              l_uint32  *pfactor)
@@ -748,10 +760,10 @@ l_uint64  limit, ratio;
  *                         Gray code conversion                        *
  *---------------------------------------------------------------------*/
 /*!
- * \brief   convertBinaryToGrayCode()
+ * \brief   convertIntToGrayCode()
  *
- * \param[in]    val
- * \return  gray code value
+ * \param[in]  val    integer value
+ * \return     corresponding gray code value
  *
  * <pre>
  * Notes:
@@ -760,20 +772,20 @@ l_uint64  limit, ratio;
  * </pre>
  */
 l_uint32
-convertBinaryToGrayCode(l_uint32 val)
+convertIntToGrayCode(l_uint32 val)
 {
     return (val >> 1) ^ val;
 }
 
 
 /*!
- * \brief   convertGrayCodeToBinary()
+ * \brief   convertGrayCodeToInt()
  *
- * \param[in]    val gray code value
- * \return  binary value
+ * \param[in]  val    gray code value
+ * \return     corresponding integer value
  */
 l_uint32
-convertGrayCodeToBinary(l_uint32 val)
+convertGrayCodeToInt(l_uint32 val)
 {
 l_uint32  shift;
 
@@ -1102,10 +1114,11 @@ L_WALLTIMER  *timer;
 char *
 l_getFormattedDate()
 {
-char        buf[sizeof "199812231952SS-08'00'"] = "", sep = 'Z';
+char        buf[128] = "", sep = 'Z';
 l_int32     gmt_offset, relh, relm;
 time_t      ut, lt;
-struct tm  *tptr;
+struct tm   Tm;
+struct tm  *tptr = &Tm;
 
     ut = time(NULL);
 
@@ -1118,7 +1131,11 @@ struct tm  *tptr;
            itself whether DST is in effect.  This is necessary because
            "gmtime" always sets "tm_isdst" to 0, which would tell
            "mktime" to presume that DST is not in effect. */
-    tptr = gmtime(&ut);
+#ifdef _WIN32
+    gmtime_s(tptr, &ut);
+#else
+    gmtime_r(&ut, tptr);
+#endif
     tptr->tm_isdst = -1;
     lt = mktime(tptr);
 
