@@ -27,9 +27,16 @@
 /*
  * writefile.c
  *
+ *     Set jpeg quality for pixWrite() and pixWriteMem()
+ *        l_int32     l_jpegSetQuality()
+ *
+ *     Set global variable LeptDebugOK for writing to named temp files
+ *        l_int32     setLeptDebugOK()
+ *
  *     High-level procedures for writing images to file:
  *        l_int32     pixaWriteFiles()
- *        l_int32     pixWrite()    [behavior depends on WRITE_AS_NAMED]
+ *        l_int32     pixWriteDebug()
+ *        l_int32     pixWrite()
  *        l_int32     pixWriteAutoFormat()
  *        l_int32     pixWriteStream()
  *        l_int32     pixWriteImpliedFormat()
@@ -52,10 +59,8 @@
  *        l_int32     pixSaveTiledWithText()
  *        void        l_chooseDisplayProg()
  *
- *     Deprecated pix output for debugging
+ *     Deprecated pix output for debugging (still used in tesseract 3.05)
  *        l_int32     pixDisplayWrite()
- *        l_int32     pixDisplayWriteFormat()
- *        l_int32     pixDisplayMultiple()
  *
  *  Supported file formats:
  *  (1) Writing is supported without any external libraries:
@@ -82,11 +87,6 @@
 #include <string.h>
 #include "allheaders.h"
 
-    /*   Special flag for pixWrite().  The default for both unix and     */
-    /*   windows is to use whatever filename is given, as opposed to     */
-    /*   insuring the filename extension matches the image compression.  */
-#define  WRITE_AS_NAMED    1
-
     /* Display program (xv, xli, xzgv, open) to be invoked by pixDisplay()  */
 #ifdef _WIN32
 static l_int32  var_DISPLAY_PROG = L_DISPLAY_WITH_IV;  /* default */
@@ -96,7 +96,7 @@ static l_int32  var_DISPLAY_PROG = L_DISPLAY_WITH_OPEN;  /* default */
 static l_int32  var_DISPLAY_PROG = L_DISPLAY_WITH_XZGV;  /* default */
 #endif  /* _WIN32 */
 
-static const l_int32  L_BUF_SIZE = 512;
+static const l_int32  L_BUFSIZE = 512;
 static const l_int32  MAX_DISPLAY_WIDTH = 1000;
 static const l_int32  MAX_DISPLAY_HEIGHT = 800;
 static const l_int32  MAX_SIZE_FOR_PNG = 200;
@@ -113,7 +113,7 @@ static const l_float32  DEFAULT_SCALING = 1.0;
     /* because that makes it static.  The 'const' in the definition of   */
     /* the array refers to the strings in the array; the ptr to the      */
     /* array is not const and can be used 'extern' in other files.)      */
-LEPT_DLL l_int32  NumImageFileFormatExtensions = 19;  /* array size */
+LEPT_DLL l_int32  NumImageFileFormatExtensions = 20;  /* array size */
 LEPT_DLL const char *ImageFileFormatExtensions[] =
          {"unknown",
           "bmp",
@@ -132,6 +132,7 @@ LEPT_DLL const char *ImageFileFormatExtensions[] =
           "jp2",
           "webp",
           "pdf",
+          "tif",
           "default",
           ""};
 
@@ -156,6 +157,71 @@ static const struct ExtensionMap extension_map[] =
                               { ".webp", IFF_WEBP      } };
 
 
+/*---------------------------------------------------------------------*
+ *           Set jpeg quality for pixWrite() and pixWriteMem()         *
+ *---------------------------------------------------------------------*/
+    /* Parameter that controls jpeg quality for high-level calls. */
+static l_int32  var_JPEG_QUALITY = 75;   /* default */
+
+/*!
+ * \brief   l_jpegSetQuality()
+ *
+ * \param[in]    new_quality    1 - 100; 75 is default; 0 defaults to 75
+ * \return       prev           previous quality
+ *
+ * <pre>
+ * Notes:
+ *      (1) This variable is used in pixWriteStream() and pixWriteMem(),
+ *          to control the jpeg quality.  The default is 75.
+ *      (2) It returns the previous quality, so for example:
+ *           l_int32  prev = l_jpegSetQuality(85);  //sets to 85
+ *           pixWriteStream(...);
+ *           l_jpegSetQuality(prev);   // resets to previous value
+ *      (3) On error, logs a message and does not change the variable.
+ */
+l_int32
+l_jpegSetQuality(l_int32  new_quality)
+{
+l_int32  prevq, newq;
+
+    PROCNAME("l_jpeqSetQuality");
+
+    prevq = var_JPEG_QUALITY;
+    newq = (new_quality == 0) ? 75 : new_quality;
+    if (newq < 1 || newq > 100)
+        L_ERROR("invalid jpeg quality; unchanged\n", procName);
+    else
+        var_JPEG_QUALITY = newq;
+    return prevq;
+}
+
+
+/*----------------------------------------------------------------------*
+ *    Set global variable LeptDebugOK for writing to named temp files   *
+ *----------------------------------------------------------------------*/
+l_int32 LeptDebugOK = 0;  /* default value */
+/*!
+ * \brief   setLeptDebugOK()
+ *
+ * \param[in]    allow     TRUE (1) or FALSE (0)
+ * \return       void
+ *
+ * <pre>
+ * Notes:
+ *      (1) This sets or clears the global variable LeptDebugOK, to
+ *          control writing files in a temp directory with names that
+ *          are compiled in.
+ *      (2) The default in the library distribution is 0.  Call with
+ *          %allow = 1 for development and debugging.
+ */
+void
+setLeptDebugOK(l_int32  allow)
+{
+    if (allow != 0) allow = 1;
+    LeptDebugOK = allow;
+}
+
+
 /*---------------------------------------------------------------------*
  *           Top-level procedures for writing images to file           *
  *---------------------------------------------------------------------*/
@@ -173,12 +239,12 @@ static const struct ExtensionMap extension_map[] =
  *          individually for each pix.
  * </pre>
  */
-l_int32
+l_ok
 pixaWriteFiles(const char  *rootname,
                PIXA        *pixa,
                l_int32      format)
 {
-char     bigbuf[L_BUF_SIZE];
+char     bigbuf[L_BUFSIZE];
 l_int32  i, n, pixformat;
 PIX     *pix;
 
@@ -199,7 +265,7 @@ PIX     *pix;
             pixformat = pixChooseOutputFormat(pix);
         else
             pixformat = format;
-        snprintf(bigbuf, L_BUF_SIZE, "%s%03d.%s", rootname, i,
+        snprintf(bigbuf, L_BUFSIZE, "%s%03d.%s", rootname, i,
                  ImageFileFormatExtensions[pixformat]);
         pixWrite(bigbuf, pix, pixformat);
         pixDestroy(&pix);
@@ -209,6 +275,39 @@ PIX     *pix;
 }
 
 
+/*!
+ * \brief   pixWriteDebug()
+ *
+ * \param[in]    fname
+ * \param[in]    pix
+ * \param[in]    format  defined in imageio.h
+ * \return  0 if OK; 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) Debug version, intended for use in the library when writing
+ *          to files in a temp directory with names that are compiled in.
+ *          This is used instead of pixWrite() for all such library calls.
+ *      (2) The global variable LeptDebugOK defaults to 0, and can be set
+ *          or cleared by the function setLeptDebugOK().
+ * </pre>
+ */
+l_ok
+pixWriteDebug(const char  *fname,
+              PIX         *pix,
+              l_int32      format)
+{
+    PROCNAME("pixWriteDebug");
+
+    if (LeptDebugOK) {
+        return pixWrite(fname, pix, format);
+    } else {
+        L_INFO("write to named temp file %s is disabled\n", procName, fname);
+        return 0;
+    }
+}
+
+
 /*!
  * \brief   pixWrite()
  *
@@ -226,17 +325,11 @@ PIX     *pix;
  *          Thanks to Dave Bryan for pointing this out.
  *      (2) If the default image format IFF_DEFAULT is requested:
  *          use the input format if known; otherwise, use a lossless format.
- *      (3) There are two modes with respect to file naming.
- *          (a) The default code writes to %fname.
- *          (b) If WRITE_AS_NAMED is defined to 0, it's a bit fancier.
- *              Then, if %fname does not have a file extension, one is
- *              automatically appended, depending on the requested format.
- *          The original intent for providing option (b) was to insure
- *          that filenames on Windows have an extension that matches
- *          the image compression.  However, this is not the default.
+ *      (3) The default jpeg quality is 75.  For some other value,
+ *          Use l_jpegSetQuality().
  * </pre>
  */
-l_int32
+l_ok
 pixWrite(const char  *fname,
          PIX         *pix,
          l_int32      format)
@@ -251,41 +344,9 @@ FILE    *fp;
     if (!fname)
         return ERROR_INT("fname not defined", procName, 1);
 
-#if  WRITE_AS_NAMED  /* Default */
-
     if ((fp = fopenWriteStream(fname, "wb+")) == NULL)
         return ERROR_INT("stream not opened", procName, 1);
 
-#else  /* Add an extension to the output name if none exists */
-
-    {l_int32  extlen;
-     char    *extension, *filebuf;
-        splitPathAtExtension(fname, NULL, &extension);
-        extlen = strlen(extension);
-        LEPT_FREE(extension);
-        if (extlen == 0) {
-            if (format == IFF_DEFAULT || format == IFF_UNKNOWN)
-                format = pixChooseOutputFormat(pix);
-
-            filebuf = (char *)LEPT_CALLOC(strlen(fname) + 10, sizeof(char));
-            if (!filebuf)
-                return ERROR_INT("filebuf not made", procName, 1);
-            strncpy(filebuf, fname, strlen(fname));
-            strcat(filebuf, ".");
-            strcat(filebuf, ImageFileFormatExtensions[format]);
-        } else {
-            filebuf = (char *)fname;
-        }
-
-        fp = fopenWriteStream(filebuf, "wb+");
-        if (filebuf != fname)
-            LEPT_FREE(filebuf);
-        if (fp == NULL)
-            return ERROR_INT("stream not opened", procName, 1);
-    }
-
-#endif  /* WRITE_AS_NAMED */
-
     ret = pixWriteStream(fp, pix, format);
     fclose(fp);
     if (ret)
@@ -301,7 +362,7 @@ FILE    *fp;
  * \param[in]    pix
  * \return  0 if OK; 1 on error
  */
-l_int32
+l_ok
 pixWriteAutoFormat(const char  *filename,
                    PIX         *pix)
 {
@@ -328,7 +389,7 @@ l_int32  format;
  * \param[in]    format
  * \return  0 if OK; 1 on error.
  */
-l_int32
+l_ok
 pixWriteStream(FILE    *fp,
                PIX     *pix,
                l_int32  format)
@@ -350,7 +411,7 @@ pixWriteStream(FILE    *fp,
         break;
 
     case IFF_JFIF_JPEG:   /* default quality; baseline sequential */
-        return pixWriteStreamJpeg(fp, pix, 75, 0);
+        return pixWriteStreamJpeg(fp, pix, var_JPEG_QUALITY, 0);
         break;
 
     case IFF_PNG:   /* no gamma value stored */
@@ -364,6 +425,7 @@ pixWriteStream(FILE    *fp,
     case IFF_TIFF_G4:        /* compressed, binary only */
     case IFF_TIFF_LZW:       /* compressed, all depths */
     case IFF_TIFF_ZIP:       /* compressed, all depths */
+    case IFF_TIFF_JPEG:      /* compressed, 8 bpp gray and 32 bpp rgb */
         return pixWriteStreamTiff(fp, pix, format);
         break;
 
@@ -420,7 +482,7 @@ pixWriteStream(FILE    *fp,
  *      (3) The jpeg default quality is 75.
  * </pre>
  */
-l_int32
+l_ok
 pixWriteImpliedFormat(const char  *filename,
                       PIX         *pix,
                       l_int32      quality,
@@ -561,7 +623,7 @@ l_int32  format = IFF_UNKNOWN;
  *          the alpha layer), and lossless png for all other situations.
  * </pre>
  */
-l_int32
+l_ok
 pixGetAutoFormat(PIX      *pix,
                  l_int32  *pformat)
 {
@@ -633,9 +695,11 @@ getFormatExtension(l_int32  format)
  *      (2) PostScript output is uncompressed, in hex ascii.
  *          Most printers support level 2 compression (tiff_g4 for 1 bpp,
  *          jpeg for 8 and 32 bpp).
+ *      (3) The default jpeg quality is 75.  For some other value,
+ *          Use l_jpegSetQuality().
  * </pre>
  */
-l_int32
+l_ok
 pixWriteMem(l_uint8  **pdata,
             size_t    *psize,
             PIX       *pix,
@@ -662,7 +726,7 @@ l_int32  ret;
         break;
 
     case IFF_JFIF_JPEG:   /* default quality; baseline sequential */
-        ret = pixWriteMemJpeg(pdata, psize, pix, 75, 0);
+        ret = pixWriteMemJpeg(pdata, psize, pix, var_JPEG_QUALITY, 0);
         break;
 
     case IFF_PNG:   /* no gamma value stored */
@@ -730,11 +794,12 @@ l_int32  ret;
  * <pre>
  * Notes:
  *      (1) This is a convenient wrapper for displaying image files.
+ *      (2) It does nothing unless LeptDebugOK == TRUE.
  *      (2) Set %scale = 0 to disable display.
  *      (3) This downscales 1 bpp to gray.
  * </pre>
  */
-l_int32
+l_ok
 l_fileDisplay(const char  *fname,
               l_int32      x,
               l_int32      y,
@@ -744,9 +809,13 @@ PIX  *pixs, *pixd;
 
     PROCNAME("l_fileDisplay");
 
+    if (!LeptDebugOK) {
+        L_INFO("displaying files is disabled; "
+               "use setLeptDebugOK(1) to enable\n", procName);
+        return 0;
+    }
     if (scale == 0.0)
         return 0;
-
     if (scale < 0.0)
         return ERROR_INT("invalid scale factor", procName, 1);
     if ((pixs = pixRead(fname)) == NULL)
@@ -780,13 +849,14 @@ PIX  *pixs, *pixd;
  *          It uses a static internal variable to number the output files
  *          written by a single process.  Behavior with a shared library
  *          may be unpredictable.
- *      (2) It uses these programs to display the image:
+ *      (2) It does nothing unless LeptDebugOK == TRUE.
+ *      (3) It uses these programs to display the image:
  *             On Unix: xzgv, xli or xv
  *             On Windows: i_view
  *          The display program must be on your $PATH variable.  It is
  *          chosen by setting the global var_DISPLAY_PROG, using
  *          l_chooseDisplayProg().  Default on Unix is xzgv.
- *      (3) Images with dimensions larger than MAX_DISPLAY_WIDTH or
+ *      (4) Images with dimensions larger than MAX_DISPLAY_WIDTH or
  *          MAX_DISPLAY_HEIGHT are downscaled to fit those constraints.
  *          This is particularly important for displaying 1 bpp images
  *          with xv, because xv automatically downscales large images
@@ -794,18 +864,18 @@ PIX  *pixs, *pixd;
  *          scale-to-gray to get decent-looking anti-aliased images.
  *          In all cases, we write a temporary file to /tmp/lept/disp,
  *          that is read by the display program.
- *      (4) The temporary file is written as png if, after initial
+ *      (5) The temporary file is written as png if, after initial
  *          processing for special cases, any of these obtain:
  *            * pix dimensions are smaller than some thresholds
  *            * pix depth is less than 8 bpp
  *            * pix is colormapped
- *      (5) For spp == 4, we call pixDisplayLayersRGBA() to show 3
+ *      (6) For spp == 4, we call pixDisplayLayersRGBA() to show 3
  *          versions of the image: the image with a fully opaque
  *          alpha, the alpha, and the image as it would appear with
  *          a white background.
  * </pre>
  */
-l_int32
+l_ok
 pixDisplay(PIX     *pixs,
            l_int32  x,
            l_int32  y)
@@ -829,7 +899,7 @@ pixDisplay(PIX     *pixs,
  *      (2) This displays the image if dispflag == 1; otherwise it punts.
  * </pre>
  */
-l_int32
+l_ok
 pixDisplayWithTitle(PIX         *pixs,
                     l_int32      x,
                     l_int32      y,
@@ -837,9 +907,9 @@ pixDisplayWithTitle(PIX         *pixs,
                     l_int32      dispflag)
 {
 char           *tempname;
-char            buffer[L_BUF_SIZE];
+char            buffer[L_BUFSIZE];
 static l_int32  index = 0;  /* caution: not .so or thread safe */
-l_int32         w, h, d, spp, maxheight, opaque, threeviews, ignore;
+l_int32         w, h, d, spp, maxheight, opaque, threeviews;
 l_float32       ratw, rath, ratmin;
 PIX            *pix0, *pix1, *pix2;
 PIXCMAP        *cmap;
@@ -852,6 +922,16 @@ char            fullpath[_MAX_PATH];
 
     PROCNAME("pixDisplayWithTitle");
 
+    if (!LeptDebugOK) {
+        L_INFO("displaying images is disabled;\n      "
+               "use setLeptDebugOK(1) to enable\n", procName);
+        return 0;
+    }
+
+#ifdef OS_IOS /* iOS 11 does not support system() */
+    return ERROR_INT("iOS 11 does not support system()", procName, 1);
+#endif /* OS_IOS */
+
     if (dispflag != 1) return 0;
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
@@ -918,10 +998,10 @@ char            fullpath[_MAX_PATH];
     index++;
     if (pixGetDepth(pix2) < 8 || pixGetColormap(pix2) ||
         (w < MAX_SIZE_FOR_PNG && h < MAX_SIZE_FOR_PNG)) {
-        snprintf(buffer, L_BUF_SIZE, "/tmp/lept/disp/write.%03d.png", index);
+        snprintf(buffer, L_BUFSIZE, "/tmp/lept/disp/write.%03d.png", index);
         pixWrite(buffer, pix2, IFF_PNG);
     } else {
-        snprintf(buffer, L_BUF_SIZE, "/tmp/lept/disp/write.%03d.jpg", index);
+        snprintf(buffer, L_BUFSIZE, "/tmp/lept/disp/write.%03d.jpg", index);
         pixWrite(buffer, pix2, IFF_JFIF_JPEG);
     }
     tempname = genPathname(buffer, NULL);
@@ -932,32 +1012,32 @@ char            fullpath[_MAX_PATH];
     if (var_DISPLAY_PROG == L_DISPLAY_WITH_XZGV) {
             /* no way to display title */
         pixGetDimensions(pix2, &wt, &ht, NULL);
-        snprintf(buffer, L_BUF_SIZE,
+        snprintf(buffer, L_BUFSIZE,
                  "xzgv --geometry %dx%d+%d+%d %s &", wt + 10, ht + 10,
                  x, y, tempname);
     } else if (var_DISPLAY_PROG == L_DISPLAY_WITH_XLI) {
         if (title) {
-            snprintf(buffer, L_BUF_SIZE,
+            snprintf(buffer, L_BUFSIZE,
                "xli -dispgamma 1.0 -quiet -geometry +%d+%d -title \"%s\" %s &",
                x, y, title, tempname);
         } else {
-            snprintf(buffer, L_BUF_SIZE,
+            snprintf(buffer, L_BUFSIZE,
                "xli -dispgamma 1.0 -quiet -geometry +%d+%d %s &",
                x, y, tempname);
         }
     } else if (var_DISPLAY_PROG == L_DISPLAY_WITH_XV) {
         if (title) {
-            snprintf(buffer, L_BUF_SIZE,
+            snprintf(buffer, L_BUFSIZE,
                      "xv -quit -geometry +%d+%d -name \"%s\" %s &",
                      x, y, title, tempname);
         } else {
-            snprintf(buffer, L_BUF_SIZE,
+            snprintf(buffer, L_BUFSIZE,
                      "xv -quit -geometry +%d+%d %s &", x, y, tempname);
         }
     } else if (var_DISPLAY_PROG == L_DISPLAY_WITH_OPEN) {
-        snprintf(buffer, L_BUF_SIZE, "open %s &", tempname);
+        snprintf(buffer, L_BUFSIZE, "open %s &", tempname);
     }
-    ignore = system(buffer);
+    callSystemDebug(buffer);
 
 #else  /* _WIN32 */
 
@@ -965,14 +1045,14 @@ char            fullpath[_MAX_PATH];
     pathname = genPathname(tempname, NULL);
     _fullpath(fullpath, pathname, sizeof(fullpath));
     if (title) {
-        snprintf(buffer, L_BUF_SIZE,
+        snprintf(buffer, L_BUFSIZE,
                  "i_view32.exe \"%s\" /pos=(%d,%d) /title=\"%s\"",
                  fullpath, x, y, title);
     } else {
-        snprintf(buffer, L_BUF_SIZE, "i_view32.exe \"%s\" /pos=(%d,%d)",
+        snprintf(buffer, L_BUFSIZE, "i_view32.exe \"%s\" /pos=(%d,%d)",
                  fullpath, x, y);
     }
-    ignore = system(buffer);
+    callSystemDebug(buffer);
     LEPT_FREE(pathname);
 
 #endif  /* _WIN32 */
@@ -995,7 +1075,7 @@ char            fullpath[_MAX_PATH];
  * \param[in]    dp depth of pixa; 8 or 32 bpp; only used on first call
  * \return  0 if OK, 1 on error.
  */
-l_int32
+l_ok
 pixSaveTiled(PIX       *pixs,
              PIXA      *pixa,
              l_float32  scalefactor,
@@ -1046,7 +1126,7 @@ pixSaveTiled(PIX       *pixs,
  *          field, which is the only field available for storing an int.
  * </pre>
  */
-l_int32
+l_ok
 pixSaveTiledOutline(PIX       *pixs,
                     PIXA      *pixa,
                     l_float32  scalefactor,
@@ -1093,7 +1173,7 @@ PIX     *pix1, *pix2, *pix3, *pix4;
         pix2 = pixClone(pix1);
     } else if (scalefactor > 1.0) {
         pix2 = pixScale(pix1, scalefactor, scalefactor);
-    } else if (scalefactor < 1.0) {
+    } else {  /* scalefactor < 1.0) */
         if (pixGetDepth(pix1) == 1)
             pix2 = pixScaleToGray(pix1, scalefactor);
         else
@@ -1176,7 +1256,7 @@ PIX     *pix1, *pix2, *pix3, *pix4;
  *      (7) See pixSaveTiledOutline() for other implementation details.
  * </pre>
  */
-l_int32
+l_ok
 pixSaveTiledWithText(PIX         *pixs,
                      PIXA        *pixa,
                      l_int32      outwidth,
@@ -1248,86 +1328,43 @@ l_chooseDisplayProg(l_int32  selection)
  * <pre>
  * Notes:
  *      (0) Deprecated.
- *      (1) This is a simple interface for writing a set of files that can
- *          be either looked at individually or saved in a pdf for viewing.
- *      (2) This defaults to jpeg output for pix that are 32 bpp or
- *          8 bpp without a colormap.  If you want to write all images
- *          losslessly, use format == IFF_PNG in pixDisplayWriteFormat().
- *      (3) See pixDisplayWriteFormat() for usage details.
- * </pre>
- */
-l_int32
-pixDisplayWrite(PIX     *pixs,
-                l_int32  reduction)
-{
-    return pixDisplayWriteFormat(pixs, reduction, IFF_DEFAULT);
-}
-
-
-/*!
- * \brief   pixDisplayWriteFormat()
- *
- * \param[in]    pix 1, 2, 4, 8, 16, 32 bpp
- * \param[in]    reduction -1 to erase and reset; 0 to disable;
- *                         otherwise this is an integer reduction factor
- * \param[in]    format IFF_DEFAULT or IFF_PNG
- * \return  0 if OK; 1 on error
- *
- * <pre>
- * Notes:
- *      (0) Deprecated.
- *      (1) This writes files with pathnames "/tmp/lept/display/file.*"
- *          if reduction > 0.  These can be collected into a pdf using
- *          pixDisplayMultiple();
- *      (2) To erase any previously written files in the output directory:
+ *      (1) This is a simple interface for writing a set of files.
+ *      (2) This uses jpeg output for pix that are 32 bpp or 8 bpp
+ *          without a colormap; otherwise, it uses png.
+ *      (3) To erase any previously written files in the output directory:
  *             pixDisplayWrite(NULL, -1);
- *      (3) If reduction > 1 and depth == 1, this does a scale-to-gray
+ *      (4) If reduction > 1 and depth == 1, this does a scale-to-gray
  *          reduction.
- *      (4) This function uses a static internal variable to number
+ *      (5) This function uses a static internal variable to number
  *          output files written by a single process.  Behavior
  *          with a shared library may be unpredictable.
- *      (5) Output file format choices are:
- *            format == IFF_DEFAULT:
- *                png if d < 8 or d == 16 or if the output pix
- *                has a colormap.   Otherwise, output is jpg.
- *            format == IFF_PNG:
- *                png (lossless) on all images.
- *      (6) For 16 bpp, the choice of full dynamic range with log scale
- *          is usually best for displaying these images.  Alternative
- *          image transforms to generate 8 bpp pix are:
+ *      (6) For 16 bpp, this displays the full dynamic range with log scale.
+ *          Alternative image transforms to generate 8 bpp pix are:
  *             pix8 = pixMaxDynamicRange(pixt, L_LINEAR_SCALE);
  *             pix8 = pixConvert16To8(pixt, 0);  // low order byte
  *             pix8 = pixConvert16To8(pixt, 1);  // high order byte
  * </pre>
  */
-l_int32
-pixDisplayWriteFormat(PIX     *pixs,
-                      l_int32  reduction,
-                      l_int32  format)
+l_ok
+pixDisplayWrite(PIX     *pixs,
+                l_int32  reduction)
 {
-char            buf[L_BUF_SIZE];
+char            buf[L_BUFSIZE];
 char           *fname;
 l_float32       scale;
 PIX            *pix1, *pix2;
 static l_int32  index = 0;  /* caution: not .so or thread safe */
 
-    PROCNAME("pixDisplayWriteFormat");
+    PROCNAME("pixDisplayWrite");
 
     if (reduction == 0) return 0;
-
     if (reduction < 0) {  /* initialize */
         lept_rmdir("lept/display");
         index = 0;
         return 0;
     }
-
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
-    if (format != IFF_DEFAULT && format != IFF_PNG) {
-        L_INFO("invalid format; using default\n", procName);
-        format = IFF_DEFAULT;
-    }
-
     if (index == 0)
         lept_mkdir("lept/display");
     index++;
@@ -1344,57 +1381,20 @@ static l_int32  index = 0;  /* caution: not .so or thread safe */
 
     if (pixGetDepth(pix1) == 16) {
         pix2 = pixMaxDynamicRange(pix1, L_LOG_SCALE);
-        snprintf(buf, L_BUF_SIZE, "file.%03d.png", index);
+        snprintf(buf, L_BUFSIZE, "file.%03d.png", index);
         fname = pathJoin("/tmp/lept/display", buf);
         pixWrite(fname, pix2, IFF_PNG);
         pixDestroy(&pix2);
-    } else if (pixGetDepth(pix1) < 8 || pixGetColormap(pix1) ||
-               format == IFF_PNG) {
-        snprintf(buf, L_BUF_SIZE, "file.%03d.png", index);
+    } else if (pixGetDepth(pix1) < 8 || pixGetColormap(pix1)) {
+        snprintf(buf, L_BUFSIZE, "file.%03d.png", index);
         fname = pathJoin("/tmp/lept/display", buf);
         pixWrite(fname, pix1, IFF_PNG);
     } else {
-        snprintf(buf, L_BUF_SIZE, "file.%03d.jpg", index);
+        snprintf(buf, L_BUFSIZE, "file.%03d.jpg", index);
         fname = pathJoin("/tmp/lept/display", buf);
-        pixWrite(fname, pix1, format);
+        pixWrite(fname, pix1, IFF_JFIF_JPEG);
     }
     LEPT_FREE(fname);
     pixDestroy(&pix1);
-
-    return 0;
-}
-
-
-/*!
- * \brief   pixDisplayMultiple()
- *
- * \param[in]    res input resolution in ppi; > 0
- * \param[in]    scalefactor scaling factor applied to each image; > 0.0
- * \param[in]    fileout pdf output file
- * \return  0 if OK; 1 on error
- *
- * <pre>
- * Notes:
- *      (0) Deprecated.
- *      (1) This is a wrapper for generating a pdf of images that have
- *          been written with pixDisplayWrite() or pixDisplayWriteFormat().
- * </pre>
- */
-l_int32
-pixDisplayMultiple(l_int32      res,
-                   l_float32    scalefactor,
-                   const char  *fileout)
-{
-    PROCNAME("pixDisplayMultiple");
-
-    if (res <= 0)
-        return ERROR_INT("invalid res", procName, 1);
-    if (scalefactor <= 0.0)
-        return ERROR_INT("invalid scalefactor", procName, 1);
-    if (!fileout)
-        return ERROR_INT("fileout not defined", procName, 1);
-
-    convertFilesToPdf("/tmp/lept/display", "file.", res, scalefactor, 0, 0,
-                      NULL, fileout);
     return 0;
 }
